/*==========================================================================
 * Copyright (c) 2003-2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// AnchorTextHarvester
//
// 03 Mar 2008 - mjh
//

#ifndef VANK_ANCHORTEXTHARVESTER_HPP
#define VANK_ANCHORTEXTHARVESTER_HPP

#include <iostream>
#include <fstream>
#include <algorithm>
#include <map>
#include "vank/ParsedDocument.hpp"
#include "vank/ObjectHandler.hpp"
#include "vank/Path.hpp"
#include "lemur/lemur-compat.hpp"
#include "lemur/Keyfile.hpp"
#include "lemur/SortMergeTextFiles.hpp"
#include "lemur/SHA1.hpp"

namespace vank
{
  namespace parse
  {
    /*! Writes anchor text from a parsed document out to a file. */
    class AnchorTextHarvester : public ObjectHandler<vank::api::ParsedDocument> {
    private:
      // holds destinationURL->(array of <sourceURL, anchor text> pairs)
      std::ofstream _linkFile;

      // docOrder file (document URL->DOCNO)
      std::ofstream _docOrder;

      char linkFileOutBuffer[lemur::file::FileMergeThread::MAX_INPUT_LINESIZE];
      char docOrderOutBuffer[lemur::file::FileMergeThread::MAX_INPUT_LINESIZE];

      lemur::file::Keyfile *_docNoKeyfile;
      lemur::file::Keyfile *_redirectKeyfile;

      lemur::utility::SHA1 SHA1Hasher;

    public:
      AnchorTextHarvester(const std::string &linkFilePath, const std::string& docOrderPath, lemur::file::Keyfile *docNoKeyfile, lemur::file::Keyfile *redirectKeyfile=NULL );
      ~AnchorTextHarvester();

      void handle( vank::api::ParsedDocument* document );

    }; // end class AnchorTextHarvester

  } // end namespace parse
} // end namespace vank



#endif // #define VANK_ANCHORTEXTHARVESTER_HPP
