/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// Annotator
//
// 19 July 2004 -- tds
//

#ifndef VANK_ANNOTATOR_HPP
#define VANK_ANNOTATOR_HPP

#include <map>
#include <vector>
#include <algorithm>
#include <set>

#include "vank/InferenceNetworkNode.hpp"
#include "vank/BeliefNode.hpp"
#include "vank/EvaluatorNode.hpp"
#include "vank/Extent.hpp"
namespace vank
{
  /*! \brief Inference net and inference net node classes. */
  namespace infnet
  {

    class Annotator : public EvaluatorNode {
    private:
      BeliefNode* _belief;
      EvaluatorNode::MResults _annotations;
      std::string _name;

      std::map<std::string, std::set<vank::api::ScoredExtentResult, vank::api::ScoredExtentResult::score_greater> > _seen;

    public:
      Annotator( const std::string& name, BeliefNode* belief );
      void add( InferenceNetworkNode* node, lemur::api::DOCID_T documentID, vank::index::Extent &extent );
      void addMatches( vank::utility::greedy_vector<vank::index::Extent>& extents, InferenceNetworkNode* node, lemur::api::DOCID_T documentID, vank::index::Extent &extent );
      void evaluate( lemur::api::DOCID_T documentID, int documentLength );

      lemur::api::DOCID_T nextCandidateDocument();
      void indexChanged( vank::index::Index& index );

      EvaluatorNode::MResults& getResults();
      const std::string& getName() const;
      const EvaluatorNode::MResults& getResults() const;
    };
  }
}

#endif // VANK_ANNOTATOR_HPP
