/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */

//
// Appliers
//
// 16 September 2004
//

#ifndef VANK_APPLIERS_HPP
#define VANK_APPLIERS_HPP

#include "vank/delete_range.hpp"
namespace vank
{
  /*! \brief Vank query language nodes and support classes. */
  namespace lang
  {

    template<class T, class NodeType = vank::lang::Node>
    class ApplySingleCopier {
    private:
      std::vector<vank::lang::Node*> _roots;
      T* _copier;

    public:
      ApplySingleCopier( std::vector<NodeType*>& previous, class vank::collection::Repository& repository ) {
        _copier = new T;

        for( size_t i=0; i<previous.size(); i++ ) {
          vank::lang::Node* root = previous[i];
          vank::lang::Node* newRoot = root->copy( *_copier );

          _roots.push_back(newRoot);
        }
      }

      ~ApplySingleCopier() {
        delete _copier;
      }

      std::vector<vank::lang::Node*>& roots() {
        return _roots;
      }
    };

    template<class T, class NodeType = vank::lang::Node>
    class ApplyCopiers {
    private:
      std::vector<vank::lang::Node*> _roots;
      std::vector<T*> _copiers;

    public:
      ApplyCopiers( std::vector<NodeType*>& previous ) {
        for( size_t i=0; i<previous.size(); i++ ) {
          vank::lang::Node* root = previous[i];
          T* copier = new T;
          vank::lang::Node* newRoot = root->copy( *copier );

          _roots.push_back(newRoot);
          _copiers.push_back(copier);
        }
      }

      ApplyCopiers( std::vector<NodeType*>& previous, class vank::collection::Repository& repository ) {
        for( size_t i=0; i<previous.size(); i++ ) {
          vank::lang::Node* root = previous[i];
          T* copier = new T( repository );
          vank::lang::Node* newRoot = root->copy( *copier );

          _roots.push_back(newRoot);
          _copiers.push_back(copier);
        }
      }

      ApplyCopiers( std::vector<NodeType*>& previous, class ListCache& listCache ) {
        for( size_t i=0; i<previous.size(); i++ ) {
          vank::lang::Node* root = previous[i];
          T* copier = new T( &listCache );
          vank::lang::Node* newRoot = root->copy( *copier );

          _roots.push_back(newRoot);
          _copiers.push_back(copier);
        }
      }

      ApplyCopiers( std::vector<NodeType*>& previous, class vank::collection::Repository& repository, class ListCache& listCache ) {
        for( size_t i=0; i<previous.size(); i++ ) {
          vank::lang::Node* root = previous[i];
          T* copier = new T( repository, listCache );
          vank::lang::Node* newRoot = root->copy( *copier );

          _roots.push_back(newRoot);
          _copiers.push_back(copier);
        }
      }

      ~ApplyCopiers() {
        vank::utility::delete_vector_contents( _copiers );
      }

      std::vector<vank::lang::Node*>& roots() {
        return _roots;
      }
    };

    template<class T, class NodeType=vank::lang::Node>
    class ApplyWalker {
    private:
      T* _walker;

    public:
      ApplyWalker( std::vector<NodeType*>& roots, T* walker ) {
        _walker = walker;
        for( size_t i=0; i<roots.size(); i++ )
          roots[i]->walk(*_walker);
      }

      T& get() {
        return _walker;
      }
    };
  }
}

#endif // VANK_APPLIERS_HPP
