/*==========================================================================
 * Copyright (c) 2012 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
*/
//
// ArabicStemmerTransformation
//

#ifndef VANK_ARABICSTEMMERTRANSFORMATION_HPP
#define VANK_ARABICSTEMMERTRANSFORMATION_HPP

#include "vank/Transformation.hpp"
#include "vank/Parameters.hpp"
#include "vank/Arabic_Stemmer_utf8.hpp"
namespace vank
{
  namespace parse
  {

    class ArabicStemmerTransformation : public Transformation {
    private:
      char* _stemBuffer;
      size_t _stemBufferSize;
      ObjectHandler<vank::api::ParsedDocument>* _handler;
      Arabic_Stemmer_utf8 *stemmer;

      char* _getBuffer( size_t length );
      const char* _getBufferEnd() const;
      vank::api::ParsedDocument* _restart( vank::api::ParsedDocument* document, size_t lastIndex, char* endOfStemming );
      vank::api::ParsedDocument* _processTerms( vank::api::ParsedDocument* document, size_t start, char* stem, const char* end );
      char* _growBuffer( size_t length, char* oldEnd );

    public:
      ArabicStemmerTransformation(vank::api::Parameters& parameters);
      ~ArabicStemmerTransformation();
      vank::api::ParsedDocument* transform( vank::api::ParsedDocument* document );

      void setHandler( ObjectHandler<vank::api::ParsedDocument>& handler );
      void handle( vank::api::ParsedDocument* document );
    };
  }
}

#endif
