/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// BeliefNode
//
// 26 January 2004 -- tds
//

#ifndef VANK_BELIEFNODE_HPP
#define VANK_BELIEFNODE_HPP

#include "vank/InferenceNetworkNode.hpp"
#include "vank/greedy_vector"
#include "vank/ScoredExtentResult.hpp"
#include <float.h>

#define VANK_HUGE_SCORE  ( DBL_MAX )
#define VANK_TINY_SCORE  ( -DBL_MAX )
namespace vank
{
  namespace infnet
  {

    class BeliefNode : public InferenceNetworkNode {
    protected:
      /// flag (and potential counter) for if belief node has siblings
      int bSiblings;

    public:
      virtual double maximumBackgroundScore() = 0;
      virtual double maximumScore() = 0;
      virtual const vank::utility::greedy_vector<vank::api::ScoredExtentResult>& score( lemur::api::DOCID_T documentID, vank::index::Extent &extent, int documentLength ) = 0;
      virtual bool hasMatch( lemur::api::DOCID_T documentID ) = 0;
      virtual const vank::utility::greedy_vector<bool>& hasMatch( lemur::api::DOCID_T documentID, const vank::utility::greedy_vector<vank::index::Extent>& extents ) = 0;
      virtual void annotate( class Annotator& annotator, lemur::api::DOCID_T documentID, vank::index::Extent &extent ) = 0;

      /// sets the siblings flag (and counter) if the belief node
      /// has siblings
      virtual void setSiblingsFlag(int f) { bSiblings=f; }
    };
  }
}

#endif // VANK_BELIEFNODE_HPP
