/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */

//
// CachedFrequencyBeliefNode
//
// 24 September 2004 -- tds
//

#ifndef VANK_CACHEDFREQUENCYBELIEFNODE_HPP
#define VANK_CACHEDFREQUENCYBELIEFNODE_HPP

#include "vank/Annotator.hpp"
#include "vank/ScoredExtentResult.hpp"
#include <string>
#include "vank/TermScoreFunction.hpp"
#include "vank/ListBeliefNode.hpp"
#include "vank/ListCache.hpp"
namespace vank
{
  namespace infnet
  {

    class CachedFrequencyBeliefNode : public BeliefNode {
    private:
      vank::query::TermScoreFunction& _function;
      vank::utility::greedy_vector<vank::api::ScoredExtentResult> _extents;
      vank::utility::greedy_vector<vank::index::DocumentContextCount>::iterator _iter;
      vank::utility::greedy_vector<bool> _matches;
      vank::lang::ListCache::CachedList* _list;
      double _maximumBackgroundScore;
      double _maximumScore;
      std::string _name;

    public:
      CachedFrequencyBeliefNode( const std::string& name,
                                 vank::lang::ListCache::CachedList* list,
                                 vank::query::TermScoreFunction& scoreFunction,
                                 double maximumBackgroundScore,
                                 double maximumScore );

      lemur::api::DOCID_T nextCandidateDocument();
      void indexChanged( vank::index::Index& index );

      double maximumBackgroundScore();
      double maximumScore();
      const vank::utility::greedy_vector<vank::api::ScoredExtentResult>& score( lemur::api::DOCID_T documentID, int begin, int end, int documentLength );
      void annotate( class Annotator& annotator, lemur::api::DOCID_T documentID, int begin, int end );
      bool hasMatch( lemur::api::DOCID_T documentID );
      const vank::utility::greedy_vector<bool>& hasMatch( lemur::api::DOCID_T documentID, const vank::utility::greedy_vector<vank::index::Extent>& extents );

      const std::string& getName() const;
    };
  }
}

#endif // VANK_CACHEDFREQUENCYBELIEFNODE_HPP
