/*==========================================================================
 * Copyright (c) 2003-2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// CompressedCollection.hpp
//
// 12 May 2004 -- tds
//

#ifndef VANK_COMPRESSEDCOLLECTION_HPP
#define VANK_COMPRESSEDCOLLECTION_HPP

#include "vank/Collection.hpp"
#include "lemur/string-set.h"
#include <string>
#include <vector>
#include "lemur/Keyfile.hpp"
#include "vank/Buffer.hpp"
#include "vank/SequentialWriteBuffer.hpp"
#include "vank/SequentialReadBuffer.hpp"
#include "vank/HashTable.hpp"
#include "vank/File.hpp"
#include "vank/Mutex.hpp"
#include "lemur/IndexTypes.hpp"
#include "vank/DeletedDocumentList.hpp"

typedef struct z_stream_s* z_stream_p;

namespace vank
{
  namespace collection
  {

    class CompressedCollection : public Collection {
    private:
      vank::thread::Mutex _lock;

      std::string _basePath;
      lemur::file::Keyfile _lookup;
      vank::file::File _storage;
      vank::file::SequentialWriteBuffer* _output;
      vank::utility::Buffer _positionsBuffer;
      z_stream_p _stream;

      vank::utility::HashTable<const char*, lemur::file::Keyfile*> _reverseLookups;
      vank::utility::HashTable<const char*, lemur::file::Keyfile*> _forwardLookups;
      String_set* _strings;

      void _writePositions( vank::api::ParsedDocument* document, int& keyLength, int& valueLength );
      void _writeMetadataItem( vank::api::ParsedDocument* document, int i, int& keyLength, int& valueLength );
      void _writeText( vank::api::ParsedDocument* document, int& keyLength, int& valueLength );
      void _writeContent( vank::api::ParsedDocument* document, int& keyLength, int& valueLength );
      void _writeContentLength( vank::api::ParsedDocument* document, int& keyLength, int& valueLength );

      void _readPositions( vank::api::ParsedDocument* document, const void* positionData, int positionDataLength );

      void _removeForwardLookups( vank::index::DeletedDocumentList& deletedList, lemur::file::Keyfile& keyfile );
      void _removeReverseLookups( vank::index::DeletedDocumentList& deletedList, lemur::file::Keyfile& keyfile );

      void _copyForwardLookup( const std::string& name,
                               lemur::file::Keyfile& other,
                               vank::index::DeletedDocumentList& deletedList,
                               lemur::api::DOCID_T documentOffset );

      void _copyReverseLookup( const std::string& name,
                               lemur::file::Keyfile& other,
                               vank::index::DeletedDocumentList& deletedList,
                               lemur::api::DOCID_T documentOffset );


      void _copyStorageEntry( vank::file::SequentialReadBuffer* input,
                              vank::file::SequentialWriteBuffer* output,
                              int key,
                              UINT64 position,
                              UINT64 length,
                              lemur::file::Keyfile& lookup );
      void _copyStorageData( vank::file::SequentialReadBuffer* input,
                             vank::file::SequentialWriteBuffer* output,
                             vank::index::DeletedDocumentList& deletedList,
                             lemur::api::DOCID_T documentOffset,
                             lemur::file::Keyfile& sourceLookup,
                             lemur::file::Keyfile& destLookup,
                             UINT64 storageLength );
      void _copyForwardLookup( const std::string& name, lemur::file::Keyfile& other, lemur::api::DOCID_T documentOffset );

      bool _storeDocs;
    public:
      CompressedCollection();
      ~CompressedCollection();

      void create( const std::string& fileName );
      void create( const std::string& fileName, const std::vector<std::string>& indexedFields );
      void create( const std::string& fileName, const std::vector<std::string>& forwardIndexedFields, const std::vector<std::string>& reverseIndexedFields,  bool storeDocs = true );
      void reopen( const std::string& fileName );
      void open( const std::string& fileName );
      void openRead( const std::string& fileName );
      void close();
      bool exists(lemur::api::DOCID_T documentID);
      vank::api::ParsedDocument* retrieve( lemur::api::DOCID_T documentID );
      std::string retrieveMetadatum( lemur::api::DOCID_T documentID, const std::string& attributeName );
      std::vector<vank::api::ParsedDocument*> retrieveByMetadatum( const std::string& attributeName, const std::string& value );
      std::vector<lemur::api::DOCID_T> retrieveIDByMetadatum( const std::string& attributeName, const std::string& value );

      void addDocument( lemur::api::DOCID_T documentID, vank::api::ParsedDocument* document );
      void compact( vank::index::DeletedDocumentList& deletedList );
      void append( vank::collection::CompressedCollection& other, vank::index::DeletedDocumentList& deletedList, lemur::api::DOCID_T documentOffset );

      std::vector<std::string> forwardFields();
      std::vector<std::string> reverseFields();
    };
  }
}

#endif // VANK_COMPRESSEDCOLLECTION_HPP
