/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// ContextCountGraphCopier
//
// 5 March 2004 -- tds
//

#ifndef VANK_CONTEXTCOUNTGRAPHCOPIER_HPP
#define VANK_CONTEXTCOUNTGRAPHCOPIER_HPP
namespace vank
{
  namespace lang
  {

    class ContextCountGraphCopier : public vank::lang::Copier {
    private:
      std::vector<vank::lang::Node*> _newNodes;
      std::vector<vank::lang::ContextCounterNode*> _contextCounters;

    public:
      ~ContextCountGraphCopier() {
        vank::utility::delete_vector_contents<vank::lang::Node*>( _newNodes );
      }

      vank::lang::Node* defaultAfter( vank::lang::Node* oldNode, vank::lang::Node* newNode ) {
        _newNodes.push_back( newNode );
        return newNode;
      }

      vank::lang::Node* after( vank::lang::RawScorerNode* oldNode, vank::lang::RawScorerNode* newNode ) {
        vank::lang::RawExtentNode* raw = newNode->getRawExtent();
        vank::lang::RawExtentNode* context = newNode->getContext();
        delete newNode;

        vank::lang::ContextCounterNode* contextCounter = new vank::lang::ContextCounterNode( raw, context );
        // need to match the name of the raw scorer we're counting for
        contextCounter->setNodeName( oldNode->nodeName() );
        _newNodes.push_back( contextCounter );
        _contextCounters.push_back( contextCounter );
        return contextCounter;
      }

      vank::lang::Node* after( vank::lang::NestedRawScorerNode* oldNode, vank::lang::NestedRawScorerNode* newNode ) {
        return after ( (vank::lang::RawScorerNode*) oldNode, (vank::lang::RawScorerNode*) newNode );
      }

      vank::lang::Node* after( vank::lang::ShrinkageScorerNode* oldNode, vank::lang::ShrinkageScorerNode* newNode ) {
        return after ( (vank::lang::RawScorerNode*) oldNode, (vank::lang::RawScorerNode*) newNode );
      }

      std::vector<vank::lang::Node*>& getNodes() {
        return _newNodes;
      }

      std::vector<vank::lang::ContextCounterNode*>& getContextCounterNodes() {
        return _contextCounters;
      }
    };

    class NoContextCountGraphCopier : public vank::lang::Copier {
    private:
      std::vector<vank::lang::Node*> _newNodes;
      std::vector<vank::lang::ContextCounterNode*> _contextCounters;

    public:
      ~NoContextCountGraphCopier() {
        vank::utility::delete_vector_contents<vank::lang::Node*>( _newNodes );
      }

      vank::lang::Node* defaultAfter( vank::lang::Node* oldNode, vank::lang::Node* newNode ) {
        _newNodes.push_back( newNode );
        return newNode;
      }

      vank::lang::Node* after( vank::lang::RawScorerNode* oldNode, vank::lang::RawScorerNode* newNode ) {
        vank::lang::RawExtentNode* raw = newNode->getRawExtent();
        vank::lang::RawExtentNode* context = newNode->getContext();
        delete newNode;
        context = 0;

        vank::lang::ContextCounterNode* contextCounter = new vank::lang::ContextCounterNode( raw, context );
        // need to match the name of the raw scorer we're counting for
        contextCounter->setNodeName( oldNode->nodeName() );
        _newNodes.push_back( contextCounter );
        _contextCounters.push_back( contextCounter );
        return contextCounter;
      }

      vank::lang::Node* after( vank::lang::NestedRawScorerNode* oldNode, vank::lang::NestedRawScorerNode* newNode ) {
        return after ( (vank::lang::RawScorerNode*) oldNode, (vank::lang::RawScorerNode*) newNode );
      }

      vank::lang::Node* after( vank::lang::ShrinkageScorerNode* oldNode, vank::lang::ShrinkageScorerNode* newNode ) {
        return after ( (vank::lang::RawScorerNode*) oldNode, (vank::lang::RawScorerNode*) newNode );
      }


      vank::lang::Node* after( vank::lang::ExtentRestriction* oldNode, vank::lang::ExtentRestriction* newNode ) {
        vank::lang::Node * child = newNode->getChild();
        delete newNode;
        return child;
      }

      std::vector<vank::lang::Node*>& getNodes() {
        return _newNodes;
      }

      std::vector<vank::lang::ContextCounterNode*>& getContextCounterNodes() {
        return _contextCounters;
      }
    };
  }
}

#endif // VANK_CONTEXTCOUNTGRAPHCOPIER_HPP
