/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// DagCopier
//
// 5 March 2004 -- tds
//

#ifndef VANK_DAGCOPIER_HPP
#define VANK_DAGCOPIER_HPP

#include <vector>
#include "vank/delete_range.hpp"
namespace vank
{
  namespace lang
  {

    class DagCopier : public vank::lang::Copier {
    private:
      std::vector<vank::lang::IndexTerm*> _terms;
      std::vector<vank::lang::Field*> _fields;
      std::vector<vank::lang::ExtentOr*> _extentOrs;
      std::vector<vank::lang::ExtentAnd*> _extentAnds;
      std::vector<vank::lang::ExtentInside*> _extentInsides;
      std::vector<vank::lang::ODNode*> _odNodes;
      std::vector<vank::lang::UWNode*> _uwNodes;
      std::vector<vank::lang::FieldWildcard*> _fieldWildcardNodes;
      std::vector<vank::lang::DocumentStructureNode*> _documentStructureNodes;

      std::vector<vank::lang::Node*> _newNodes;

      template<class T>
      T* _findReplacement( std::vector<T*>& replacements, T* candidate ) {
        T* replacement = 0;

        for( unsigned int i=0; i<replacements.size(); i++ ) {
          if( (*candidate) == (*replacements[i]) ) {
            replacement = replacements[i];
            break;
          }
        }

        if( replacement ) {
          delete candidate;
          candidate = replacement;
        } else {
          _newNodes.push_back( candidate );
          replacements.push_back( candidate );
        }

        return candidate;
      }

    public:
      ~DagCopier() {
        vank::utility::delete_vector_contents( _newNodes );
      }

      vank::lang::Node* defaultAfter( vank::lang::Node* oldNode, vank::lang::Node* newNode ) {
        _newNodes.push_back( newNode );
        return newNode;
      }

      vank::lang::Node* after( vank::lang::IndexTerm* indexTerm, vank::lang::IndexTerm* newIndexTerm ) {
        return _findReplacement<vank::lang::IndexTerm>( _terms, newIndexTerm );
      }

      vank::lang::Node* after( vank::lang::Field* field, vank::lang::Field* newField ) {
        return _findReplacement<vank::lang::Field>( _fields, newField );
      }

      vank::lang::Node* after( vank::lang::ExtentOr* oldExtentOr, vank::lang::ExtentOr* newExtentOr ) {
        return _findReplacement<vank::lang::ExtentOr>( _extentOrs, newExtentOr );
      }

      vank::lang::Node* after( vank::lang::ExtentAnd* oldExtentAnd, vank::lang::ExtentAnd* newExtentAnd ) {
        return _findReplacement<vank::lang::ExtentAnd>( _extentAnds, newExtentAnd );
      }

      vank::lang::Node* after( vank::lang::ExtentInside* oldExtentInside, vank::lang::ExtentInside* newExtentInside ) {
        return _findReplacement<vank::lang::ExtentInside>( _extentInsides, newExtentInside );
      }

      vank::lang::Node* after( vank::lang::NestedExtentInside* oldExtentInside, vank::lang::NestedExtentInside* newExtentInside ) {
        return after((vank::lang::ExtentInside*) oldExtentInside, (vank::lang::ExtentInside*) newExtentInside );
      }

      vank::lang::Node* after( vank::lang::ODNode* oldODNode, vank::lang::ODNode* newODNode ) {
        return _findReplacement<vank::lang::ODNode>( _odNodes, newODNode );
      }

      vank::lang::Node* after( vank::lang::UWNode* oldUWNode, vank::lang::UWNode* newUWNode ) {
        return _findReplacement<vank::lang::UWNode>( _uwNodes, newUWNode );
      }

      vank::lang::Node* after( vank::lang::FieldWildcard* fieldWildcard, vank::lang::FieldWildcard* newFieldWildcard ) {
        return _findReplacement<vank::lang::FieldWildcard>( _fieldWildcardNodes, newFieldWildcard );
      }

      vank::lang::Node* after( vank::lang::DocumentStructureNode* docStruct, vank::lang::DocumentStructureNode* newDocStruct ) {
        return _findReplacement<vank::lang::DocumentStructureNode>( _documentStructureNodes, newDocStruct );
      }

    };
  }
}

#endif // VANK_DAGCOPIER_HPP
