/*==========================================================================
 * Copyright (c) 2005 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */

//
// DeletedDocumentList
//
// 3 February 2005 -- tds
//
// While vector<bool> might have been a nice clean choice for this
// bitmap, there was no platform-independent way to just blit it
// to disk.  I decided to roll my own using a Buffer.
//

#ifndef VANK_DELETEDDOCUMENTLIST_HPP
#define VANK_DELETEDDOCUMENTLIST_HPP

#include <vector>
#include "vank/ReadersWritersLock.hpp"
#include "vank/ReaderLockable.hpp"
#include "vank/WriterLockable.hpp"
#include "vank/Buffer.hpp"
#include "lemur/IndexTypes.hpp"

namespace vank
{
  namespace index
  {

    class DeletedDocumentList {
    private:
      bool _modified;
      vank::thread::ReadersWritersLock _lock;
      vank::thread::ReaderLockable _readLock;
      vank::thread::WriterLockable _writeLock;
      UINT64 _deletedCount;

      vank::utility::Buffer _bitmap;
      void _grow( lemur::api::DOCID_T documentID );
      void _calculateDeletedCount();

    public:
      class read_transaction {
      private:
        vank::thread::ReadersWritersLock& _lock;
        vank::utility::Buffer& _bitmap;

      public:
        read_transaction( DeletedDocumentList& list );
        ~read_transaction();

        lemur::api::DOCID_T nextCandidateDocument( lemur::api::DOCID_T documentID );
        bool isDeleted( lemur::api::DOCID_T documentID ) const;
      };

      DeletedDocumentList();

      void append( DeletedDocumentList& other, int documentCount );
      void markDeleted( lemur::api::DOCID_T documentID );
      bool isDeleted( lemur::api::DOCID_T documentID );
      UINT64 deletedCount() const;
      read_transaction* getReadTransaction();

      void read( const std::string& filename );
      void write( const std::string& filename );
    };
  }
}

#endif // VANK_DELETEDDOCUMENTLIST_HPP
