/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */

//
// DiskDocListIterator
//
// 10 December 2004 -- tds
//

#ifndef VANK_DISKDOCLISTITERATOR_HPP
#define VANK_DISKDOCLISTITERATOR_HPP

#include "vank/DocListIterator.hpp"
#include "vank/SequentialReadBuffer.hpp"
#include "lemur/Keyfile.hpp"

namespace vank {
  namespace index {
    class DiskDocListIterator : public DocListIterator {
    private:
      const char* _list;
      const char* _listEnd;
      lemur::api::DOCID_T _skipDocument;

      vank::file::SequentialReadBuffer* _file;
      UINT64 _startOffset;
      UINT64 _endOffset;
      bool _hasTopdocs;
      bool _isFrequent;

      vank::utility::greedy_vector<TopDocument> _topdocs;
      DocumentData _data;
      DocumentData* _result;

      TermData* _termData;
      bool _ownTermData;
      char _term[ lemur::file::Keyfile::MAX_KEY_LENGTH+1 ];
      int _fieldCount;

      void _readEntry();
      void _readSkip();
      void _readTopdocs();
      void _readTermData( int headerLength );

    public:
      DiskDocListIterator( vank::file::SequentialReadBuffer* buffer, UINT64 startOffset, int fieldCount );
      ~DiskDocListIterator();
      void setStartOffset( UINT64 startOffset, TermData* termData );

      const vank::utility::greedy_vector<TopDocument>& topDocuments();

      void startIteration();
      bool nextEntry();
      bool nextEntry( lemur::api::DOCID_T documentID );
      DocumentData* currentEntry();
      bool finished();
      bool isFrequent() const;
      TermData* termData();
    };
  }
}

#endif // VANK_DISKDOCLISTITERATOR_HPP
