/*==========================================================================
 * Copyright (c) 2005 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */

//
// DiskKeyfileVocabularyIterator
//
// 20 January 2005 -- tds
//

#ifndef VANK_DISKKEYFILEVOCABULARYITERATOR_HPP
#define VANK_DISKKEYFILEVOCABULARYITERATOR_HPP

#include "vank/DiskTermData.hpp"
#include "lemur/Keyfile.hpp"
#include "vank/VocabularyIterator.hpp"
#include "vank/Mutex.hpp"
#include "vank/BulkTree.hpp"

namespace vank {
  namespace index {
    class DiskKeyfileVocabularyIterator : public VocabularyIterator {
    private:
      DiskTermData* _diskTermData;
      int _baseID;
      vank::file::BulkTreeReader& _bulkTree;
      vank::file::BulkTreeIterator* _bulkIterator;
      int _fieldCount;

      char _termString[1024];

      vank::utility::Buffer _compressedData;
      vank::utility::Buffer _decompressedData;

      vank::thread::Mutex& _mutex;
      bool _holdingLock;
      bool _finished;

      void _acquire();
      void _release();
      bool _readData();

      // this tells us if the last nextEntry() came from
      // a start iteration or not - needed for nextEntry(const char*)
      // call
      bool _justStartedIteration;

    public:
      DiskKeyfileVocabularyIterator( int baseID, vank::file::BulkTreeReader& bulkTree, vank::thread::Mutex& mutex, int fieldCount );
      ~DiskKeyfileVocabularyIterator();

      void startIteration();
      bool nextEntry();
      bool nextEntry(const char *skipTo);
      vank::index::DiskTermData* currentEntry();
      bool finished();
    };
  }
}

#endif // VANK_DISKKEYFILEVOCABULARYITERATOR_HPP
