/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */

//
// DiskTermData
//
// 13 December 2004 -- tds
//

#ifndef VANK_DISKTERMDATA_HPP
#define VANK_DISKTERMDATA_HPP

#include "vank/TermData.hpp"
#include "lemur/Keyfile.hpp"
#include "lemur/IndexTypes.hpp"

namespace vank {
  namespace index {
    struct DiskTermData {
      enum {
        WithOffsets = 0x01,
        WithString = 0x02,
        WithTermID = 0x04
      };

      TermData* termData;
      lemur::api::TERMID_T termID;

      UINT64 startOffset;
      UINT64 length;
    };
  }
}


//
// disktermdata_decompress
//

inline void disktermdata_compress( vank::utility::RVLCompressStream& stream, vank::index::DiskTermData* diskData, int fieldCount, int mode ) {
  ::termdata_compress( stream, diskData->termData, fieldCount );

  if( mode & vank::index::DiskTermData::WithTermID ) {
    stream << diskData->termID;
  }

  if( mode & vank::index::DiskTermData::WithString ) {
    stream << diskData->termData->term;
  }

  if( mode & vank::index::DiskTermData::WithOffsets ) {
    stream << diskData->startOffset
           << diskData->length;
  }
}

//
// disktermdata_decompress
//

inline vank::index::DiskTermData* disktermdata_decompress( vank::utility::RVLDecompressStream& stream, void* buffer, int fieldCount, int mode ) {
  vank::index::DiskTermData* diskData = (vank::index::DiskTermData*) buffer;

  int termDataSize = ::termdata_size( fieldCount );
  char* termLocation = (char*)buffer + sizeof(vank::index::DiskTermData) + termDataSize;
  vank::index::TermData* termDataLocation = (vank::index::TermData*) ((char*)buffer + sizeof(vank::index::DiskTermData));

  diskData->termData = termDataLocation;
  diskData->termData->term = termLocation;

  // set first byte of string to zero
  termLocation[0] = 0;

  ::termdata_decompress( stream, diskData->termData, fieldCount );

  if( mode & vank::index::DiskTermData::WithTermID ) {
    stream >> diskData->termID;
  } else {
    diskData->termID = 0;
  }

  if( mode & vank::index::DiskTermData::WithString ) {
    stream >> termLocation;
  }

  if( mode & vank::index::DiskTermData::WithOffsets ) {
    stream >> diskData->startOffset
           >> diskData->length;
  } else {
    diskData->startOffset = 0;
    diskData->length = 0;
  }

  return diskData;
}

//
// disktermdata_size
//

inline int disktermdata_size( int fieldCount ) {
  // how much space are we going to need?
  int termDataSize = ::termdata_size( fieldCount );
  int totalSize = termDataSize + (lemur::file::Keyfile::MAX_KEY_LENGTH+2) + sizeof(vank::index::DiskTermData);

  return totalSize;
}

//
// disktermdata_create
//

inline vank::index::DiskTermData* disktermdata_create( int fieldCount ) {
  char* dataBlock = (char*) malloc( disktermdata_size( fieldCount ) );

  vank::index::DiskTermData* diskTermData = (vank::index::DiskTermData*) dataBlock;

  diskTermData->termData = (vank::index::TermData*) (dataBlock +
                                                      sizeof (vank::index::DiskTermData));
  termdata_construct( diskTermData->termData, fieldCount );

  diskTermData->termData->term = dataBlock +
    sizeof (vank::index::DiskTermData) +
    termdata_size( fieldCount );
  const_cast<char*>(diskTermData->termData->term)[0] = 0;

  return diskTermData;
}

//
// disktermdata_decompress
//

inline vank::index::DiskTermData* disktermdata_decompress( vank::utility::RVLDecompressStream& stream, int fieldCount, int mode ) {
  // how much space are we going to need?
  int totalSize = disktermdata_size( fieldCount );

  return ::disktermdata_decompress( stream, malloc( totalSize ), fieldCount, mode );
}

//
// disktermdata_delete
//

inline void disktermdata_delete( vank::index::DiskTermData* diskData ) {
  free( diskData );
}

#endif // VANK_DISKTERMDATA_HPP
