/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
*/


//
// DocListMemoryBuilder.hpp
//
// tds - 17 December 2003
//

#ifndef LEMUR_KEYFILEDOCLISTMEMORYBUILDER_HPP
#define LEMUR_KEYFILEDOCLISTMEMORYBUILDER_HPP

#include "lemur/RVLCompress.hpp"
#include <vector>
#include <assert.h>
#include "vank/greedy_vector"
#include "vank/DocListIterator.hpp"
#include "vank/RegionAllocator.hpp"

namespace vank {
  namespace index {
    struct DocListMemoryBuilderSegment {
      DocListMemoryBuilderSegment( char* b, char* d, char* c ) {
        base = b;
        data = d;
        capacity = c;
      }

      char* base;
      char* data;
      char* capacity;
    };

    class DocListMemoryBuilderIterator : public DocListIterator {
      const vank::utility::greedy_vector< DocListMemoryBuilderSegment, 4 >* _lists;
      vank::utility::greedy_vector< DocListMemoryBuilderSegment, 4 >::const_iterator _current;
      vank::index::DocListIterator::DocumentData _data;
      vank::utility::greedy_vector<DocListIterator::TopDocument> _emptyTopDocuments;

      const char* _list;
      const char* _listEnd;
      bool _finished;

      TermData* _termData;

    public:
      DocListMemoryBuilderIterator();
      DocListMemoryBuilderIterator( class DocListMemoryBuilder& builder, TermData* termData );

      void reset( class DocListMemoryBuilder& builder, TermData* termData );
      void reset( const vank::utility::greedy_vector< DocListMemoryBuilderSegment, 4 >& lists, TermData* termData );

      void startIteration();
      bool finished();
      bool nextEntry( lemur::api::DOCID_T documentID );
      bool nextEntry();
      TermData* termData();
      DocListIterator::DocumentData* currentEntry();
      vank::utility::greedy_vector<DocListIterator::TopDocument>& topDocuments();
    };

    class DocListMemoryBuilder {
    public:
      typedef DocListMemoryBuilderIterator iterator;
      friend class DocListMemoryBuilderIterator;

    private:
      int _documentFrequency;
      int _termFrequency;

      vank::utility::greedy_vector< DocListMemoryBuilderSegment, 4 > _lists;

      char* _list;
      char* _listBegin;
      char* _listEnd;

      char* _documentPointer;
      char* _locationCountPointer;

      int _lastLocation;
      int _lastDocument;
      int _lastTermFrequency;

      vank::utility::RegionAllocator* _allocator;

      inline void _safeAddLocation( int position );
      size_t _roundUp( size_t amount );
      void _grow();
      void _terminateDocument();

    public:
      DocListMemoryBuilder( vank::utility::RegionAllocator* allocator );
      ~DocListMemoryBuilder();
      const DocListMemoryBuilder& operator=( DocListMemoryBuilder& other );

      void startDocument( int docID );
      void addLocation( int location );
      void endDocument();

      void clear();
      void flush();
      bool empty();

      int documentFrequency() const;
      int termFrequency() const;
      size_t memorySize() const;
    };
  }
}

#endif // LEMUR_DOCLISTMEMORYBUILDER_HPP
