/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
*/


//
// DocumentStructureHolderNode
//
// 18 June 2005 -- pto
//
// This node stores the document structure for the current document
// to be shared among shrinkage belief nodes or other nodes that
// may need to use the document structure for retrieval.
//

#ifndef VANK_DOCUMENTSTRUCTUREHOLDERNODE_HPP
#define VANK_DOCUMENTSTRUCTUREHOLDERNODE_HPP

#include "vank/ListIteratorNode.hpp"
#include "vank/greedy_vector"
#include "vank/Index.hpp"
#include "vank/DocumentStructure.hpp"

namespace vank {
  namespace infnet {

    class DocumentStructureHolderNode : public InferenceNetworkNode {
    private:
      vank::index::Index * _index;
      lemur::api::DOCID_T _nextDocument;
      vank::index::TermListFileIterator * _docIter;
      lemur::api::DOCID_T  _docIterID;
      std::string _name;
      vank::index::DocumentStructure * _documentStructure;

    public:
      DocumentStructureHolderNode( const std::string& name );
      ~DocumentStructureHolderNode();

      void prepare( lemur::api::DOCID_T documentID );
      lemur::api::DOCID_T nextCandidateDocument();
      const std::string& getName() const;
      vank::index::DocumentStructure * getDocumentStructure();

      void indexChanged( vank::index::Index& index);
    };
  }
}
#endif // VANK_DOCUMENTSTRUCTUREHOLDERNODE_HPP
