/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// DocumentVector
//
// An independent representation of a document term list,
// suitable for network transmission.
//

#ifndef VANK_DOCUMENTVECTOR_HPP
#define VANK_DOCUMENTVECTOR_HPP

#include <string>
#include <map>
#include <vector>
#include "vank/Index.hpp"
namespace vank
{
  /*! \brief Vank API classes for interacting with vank collections. */
  namespace api
  {

    class DocumentVector {
    public:
      struct Field {
        std::string name;
        int begin;
        int end;
        INT64 number;
        int ordinal;
        int parentOrdinal;

        Field() { }
        Field(const vank::index::FieldExtent &f) {
          begin=f.begin;
          end=f.end;
          number=f.number;
          ordinal=f.ordinal;
          parentOrdinal=f.parentOrdinal;
        }
      };

    private:
      std::vector<std::string> _stems;
      std::vector<int> _positions;
      std::vector<Field> _fields;

      void _init( vank::index::Index* index, const class vank::index::TermList* termList, std::map<int,std::string>* termStringMap );

    public:
      DocumentVector();
      DocumentVector( vank::index::Index* index, const class vank::index::TermList* termList );
      DocumentVector( vank::index::Index* index, const class vank::index::TermList* termList, std::map<int,std::string>& termStringMap );

      std::vector<std::string>& stems();
      const std::vector<std::string>& stems() const;

      std::vector<int>& positions();
      const std::vector<int>& positions() const;

      std::vector<Field>& fields();
      const std::vector<Field>& fields() const;
    };
  }
}

#endif // VANK_DOCUMENTVECTOR_HPP
