/*==========================================================================
 * Copyright (c) 2006 Carnegie Mellon University.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software (and below), and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */

//
// ExtentDescendantNode
//
// 31 Jan 2006 - pto
//
//
// Checks to make sure the inner node has an extent that matches the extent
// of a descendant of nodes whose extents matching the outer extent
//

#ifndef VANK_EXTENTDESCENDANTNODE_HPP
#define VANK_EXTENTDESCENDANTNODE_HPP

#include "vank/ListIteratorNode.hpp"
#include "vank/DocumentStructureHolderNode.hpp"
#include "vank/greedy_vector"
#include "vank/Extent.hpp"
namespace vank
{
  namespace infnet
  {

    class ExtentDescendantNode : public ListIteratorNode {
      ListIteratorNode* _inner;
      ListIteratorNode* _outer;
      DocumentStructureHolderNode & _docStructHolder;
      vank::utility::greedy_vector<vank::index::Extent> _extents;
      std::string _name;


      std::set<int> _ancestors;
      std::set<int> _leafs;


    public:
      ExtentDescendantNode( const std::string& name,
                            ListIteratorNode* inner,
                            ListIteratorNode* outer,
                            DocumentStructureHolderNode & documentStructureHolderNode  );

      void prepare( lemur::api::DOCID_T documentID );
      const vank::utility::greedy_vector<vank::index::Extent>& extents();
      lemur::api::DOCID_T nextCandidateDocument();
      void indexChanged( vank::index::Index& index );

      const std::string& getName() const;
      void annotate( class Annotator& annotator, lemur::api::DOCID_T documentID, vank::index::Extent &extent );

      const vank::utility::greedy_vector<vank::index::Extent>& matches( vank::index::Extent &extent );
    };
  }
}

#endif // VANK_EXTENTDESCENDANTNODE_HPP
