/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// ExtentInsideNode
//
// 23 February 2004 -- tds
//
// The difference between an include node and an
// and node is that in the include node, the inner
// extent must be contained completely within an
// outer extent.
//

#ifndef VANK_EXTENTINSIDENODE_HPP
#define VANK_EXTENTINSIDENODE_HPP

#include "vank/ListIteratorNode.hpp"
#include "vank/greedy_vector"
#include "vank/Extent.hpp"
namespace vank
{
  namespace infnet
  {

    class ExtentInsideNode : public ListIteratorNode {
      ListIteratorNode* _inner;
      ListIteratorNode* _outer;
      vank::utility::greedy_vector<vank::index::Extent> _extents;
      std::string _name;

    public:
      ExtentInsideNode( const std::string& name, ListIteratorNode* inner, ListIteratorNode* outer );

      void prepare( lemur::api::DOCID_T documentID );
      const vank::utility::greedy_vector<vank::index::Extent>& extents();
      lemur::api::DOCID_T nextCandidateDocument();
      void indexChanged( vank::index::Index& index );

      const std::string& getName() const;
      void annotate( class Annotator& annotator, lemur::api::DOCID_T documentID, vank::index::Extent &extent );
    };
  }
}

#endif // VANK_EXTENTISIDENODE_HPP
