/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// ExtentRestrictionModelAnnotatorCopier
//
// 16 August 2004 -- tds
//
// This copier annotates RawScorerNodes with the language models
// of the surrounding ExtentRestrictions.
//
// For example, the query #combine[sentence]( dog cat )
// should be synonymous with #combine[sentence]( dog.(sentence) cat.(sentence) ).
// We push the sentence language model down the tree and attach it to "dog" and "cat".
//

#ifndef VANK_EXTENTRESTRICTIONMODELANNOTATORCOPIER_HPP
#define VANK_EXTENTRESTRICTIONMODELANNOTATORCOPIER_HPP

#include <stack>
#include <vector>
#include "vank/QuerySpec.hpp"
namespace vank
{
  namespace lang
  {

    class ExtentRestrictionModelAnnotatorCopier : public vank::lang::Copier {
    private:
      std::vector<vank::lang::Node*> _nodes;
      std::stack< vank::lang::ExtentRestriction* > _restrictions;

    public:
      ~ExtentRestrictionModelAnnotatorCopier() {
        vank::utility::delete_vector_contents( _nodes );
      }

      vank::lang::Node* defaultAfter( vank::lang::Node* old, vank::lang::Node* newNode ) {
        _nodes.push_back( newNode );
        return newNode;
      }

      void before( vank::lang::ExtentRestriction* old ) {
        _restrictions.push( old );
      }

      void before( vank::lang::ExtentEnforcement* old ) {
        before( (vank::lang::ExtentRestriction*) old );
      }

      vank::lang::Node* after( vank::lang::ExtentRestriction* oldNode, vank::lang::ExtentRestriction* newNode ) {
        _restrictions.pop();
        _nodes.push_back( newNode );
        return newNode;
      }

      vank::lang::Node* after( vank::lang::ExtentEnforcement* oldNode, vank::lang::ExtentEnforcement* newNode ) {
        return after( (vank::lang::ExtentRestriction*) oldNode, (vank::lang::ExtentRestriction*) newNode );
      }

      vank::lang::Node* after( vank::lang::RawScorerNode* oldNode, vank::lang::RawScorerNode* newNode ) {
        if( newNode->getContext() == 0 && _restrictions.size() ) {
          newNode->setContext( _restrictions.top()->getField() );
        }
        _nodes.push_back( newNode ); // should track for free.
        return newNode;
      }

      vank::lang::Node* after( vank::lang::NestedRawScorerNode* oldNode, vank::lang::NestedRawScorerNode* newNode ) {
        return after( (vank::lang::RawScorerNode*) oldNode, (vank::lang::RawScorerNode*) newNode );
      }
    };
  }
}

#endif // VANK_EXTENTRESTRICTIONMODELANNOTATORCOPIER_HPP
