/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// ExtentRestrictionNode
//
// 6 July 2004 -- tds
//

#ifndef VANK_EXTENTRESTRICTIONNODE_HPP
#define VANK_EXTENTRESTRICTIONNODE_HPP

#include "vank/BeliefNode.hpp"
#include "vank/FieldIteratorNode.hpp"
#include "vank/greedy_vector"
#include "vank/Extent.hpp"
namespace vank
{
  namespace infnet
  {

    class ExtentRestrictionNode : public BeliefNode {
    private:
      BeliefNode* _child;
      ListIteratorNode* _field;
      vank::utility::greedy_vector<vank::api::ScoredExtentResult> _scores;
      vank::utility::greedy_vector<bool> _matches;
      std::string _name;

    public:
      ExtentRestrictionNode( const std::string& name, BeliefNode* child, ListIteratorNode* field );

      lemur::api::DOCID_T nextCandidateDocument();
      void indexChanged( vank::index::Index& index );

      double maximumScore();
      double maximumBackgroundScore();

      const vank::utility::greedy_vector<vank::api::ScoredExtentResult>& score( lemur::api::DOCID_T documentID, vank::index::Extent &extent, int documentLength );
      void annotate( class Annotator& annotator, lemur::api::DOCID_T documentID, vank::index::Extent &extent);
      bool hasMatch( lemur::api::DOCID_T documentID );
      const vank::utility::greedy_vector<bool>& hasMatch( lemur::api::DOCID_T documentID, const vank::utility::greedy_vector<vank::index::Extent>& extents );
      const std::string& getName() const;

      /// over-ridden to set child nodes too
      void setSiblingsFlag(int f);
    };
  }
}

#endif // VANK_EXTENTRESTRICTIONNODE_HPP
