/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// FilterRequireNode
//
// 13 April 2004 -- tds
//

#ifndef VANK_FILTERREQUIRENODE_HPP
#define VANK_FILTERREQUIRENODE_HPP

#include "vank/BeliefNode.hpp"
#include "vank/ListIteratorNode.hpp"
#include "vank/Extent.hpp"
namespace vank
{
  namespace infnet
  {

    class FilterRequireNode : public BeliefNode {
    private:
      ListIteratorNode* _filter;
      BeliefNode* _required;
      vank::utility::greedy_vector<vank::api::ScoredExtentResult> _extents;
      vank::utility::greedy_vector<bool> _matches;
      std::string _name;

    public:
      FilterRequireNode( const std::string& name, ListIteratorNode* filter,
                         BeliefNode* required );

      double maximumBackgroundScore();
      double maximumScore();
      const vank::utility::greedy_vector<vank::api::ScoredExtentResult>& score( lemur::api::DOCID_T documentID, vank::index::Extent &extent, int documentLength );
      bool hasMatch( lemur::api::DOCID_T documentID );
      const vank::utility::greedy_vector<bool>& hasMatch( lemur::api::DOCID_T documentID, const vank::utility::greedy_vector<vank::index::Extent>& extents );

      lemur::api::DOCID_T nextCandidateDocument();
      void indexChanged( vank::index::Index& index );

      const std::string& getName() const;
      void annotate( class Annotator& annotator, lemur::api::DOCID_T documentID, vank::index::Extent &extent );

      virtual void setSiblingsFlag(int f){
        bSiblings=f; // need to set the flag for the current node itself.
        if (_required) {  _required->setSiblingsFlag(f); }
      }

    };
  }
}

#endif // VANK_FILTERREQUIRENODE_HPP
