/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */
//
// FixedPassageNode
//
// 23 February 2005 -- tds
//

#ifndef VANK_FIXEDPASSAGENODE_HPP
#define VANK_FIXEDPASSAGENODE_HPP

#include "vank/BeliefNode.hpp"
#include "vank/FieldIteratorNode.hpp"
#include "vank/greedy_vector"
#include "vank/Extent.hpp"
namespace vank
{
  namespace infnet
  {

    class FixedPassageNode : public BeliefNode {
    private:
      BeliefNode* _child;
      vank::utility::greedy_vector<vank::api::ScoredExtentResult> _scores;
      vank::utility::greedy_vector<bool> _matches;
      vank::utility::greedy_vector<vank::index::Extent> _subextents;

      std::string _name;

      int _windowSize;
      int _increment;
      void _addSubextents( const vank::index::Extent& extent );
      void _buildSubextents( const vank::index::Extent& extent );
      void _buildSubextents( const vank::utility::greedy_vector<vank::index::Extent>& extents );

    public:
      FixedPassageNode( const std::string& name, BeliefNode* child, int windowSize, int increment );

      lemur::api::DOCID_T nextCandidateDocument();
      void indexChanged( vank::index::Index& index );

      double maximumScore();
      double maximumBackgroundScore();

      const vank::utility::greedy_vector<vank::api::ScoredExtentResult>& score( lemur::api::DOCID_T documentID, vank::index::Extent &extent, int documentLength );
      void annotate( class Annotator& annotator, lemur::api::DOCID_T documentID, vank::index::Extent &extent );

      const vank::utility::greedy_vector<bool>& hasMatch( lemur::api::DOCID_T documentID, const vank::utility::greedy_vector<vank::index::Extent>& extents );
      bool hasMatch( lemur::api::DOCID_T documentID );
      const std::string& getName() const;

      virtual void setSiblingsFlag(int f){
        bSiblings=f; // need to set the flag for the current node itself.
        if (_child) {  _child->setSiblingsFlag(f); }
      }

    };
  }
}


#endif // VANK_FIXEDPASSAGENODE_HPP
