/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */

//
// InferenceNetwork
//
// 24 February 2004 -- tds
//

#ifndef VANK_INFERENCENETWORK_HPP
#define VANK_INFERENCENETWORK_HPP

#include "vank/BeliefNode.hpp"
#include "vank/EvaluatorNode.hpp"
#include "vank/ListIteratorNode.hpp"
#include "vank/TermScoreFunction.hpp"
#include "vank/Repository.hpp"
#include "vank/Index.hpp"
#include "vank/DeletedDocumentList.hpp"
#include "vank/PriorListIterator.hpp"
#include "vank/DocumentStructureHolderNode.hpp"

namespace vank
{
  namespace infnet
  {
    class InferenceNetwork {
    public:
      typedef std::map< std::string, EvaluatorNode::MResults > MAllResults;

      //
      // MAllResults stores results indexed first by node name, then second by the node's
      // result name.  For instance, to retrieve occurrence counts from a
      // ContextCountAccumulator named "bd45a", you'd type 'results["bd45a"]["occurrences"]'.
      //

    private:
      std::vector<std::string> _termNames;
      std::vector<std::string> _fieldNames;
      std::vector<std::string> _priorNames;

      std::vector<class vank::index::DocExtentListIterator*> _fieldIterators;
      std::vector<class vank::index::DocListIterator*> _docIterators;
      std::vector<class vank::collection::PriorListIterator*> _priorIterators;
      std::vector<ListIteratorNode*> _listIteratorNodes;
      std::vector<BeliefNode*> _beliefNodes;
      std::vector<EvaluatorNode*> _evaluators;
      std::vector<EvaluatorNode*> _complexEvaluators;
      std::vector<vank::query::TermScoreFunction*> _scoreFunctions;

      DocumentStructureHolderNode * _documentStructureHolderNode;

      vank::utility::greedy_vector<class vank::index::DocListIterator*> _closeIterators;
      int _closeIteratorBound;

      vank::collection::Repository& _repository;
      MAllResults _results;

      void _indexChanged( vank::index::Index& index );
      void _indexFinished( vank::index::Index& index );

      void _moveToDocument( lemur::api::DOCID_T candidate );
      void _moveDocListIterators( lemur::api::DOCID_T candidate );

      lemur::api::DOCID_T _nextCandidateDocument( vank::index::DeletedDocumentList::read_transaction* deleted );
      void _evaluateDocument( vank::index::Index& index, lemur::api::DOCID_T document );
      void _evaluateIndex( vank::index::Index& index );

    public:
      InferenceNetwork( vank::collection::Repository& repository );
      ~InferenceNetwork();

      const std::vector<EvaluatorNode*>& getEvaluators() const;

      vank::index::DocListIterator* getDocIterator( int index );
      vank::index::DocExtentListIterator* getFieldIterator( int index );
      vank::collection::PriorListIterator* getPriorIterator( int index );

      int addDocIterator( const std::string& term );
      int addFieldIterator( const std::string& field );
      int addPriorIterator( const std::string& prior );

      void addListNode( ListIteratorNode* listNode );
      void addBeliefNode( BeliefNode* beliefNode );
      void addEvaluatorNode( EvaluatorNode* evaluatorNode );
      void addComplexEvaluatorNode( EvaluatorNode* complexEvaluator );
      void addScoreFunction( vank::query::TermScoreFunction* scoreFunction );
      void addDocumentStructureHolderNode( DocumentStructureHolderNode* docStruct );
      const MAllResults& evaluate();
    };
  }
}

#endif // VANK_INFERENCENETWORK_HPP
