/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// InferenceNetworkBuilder
//
// 8 March 2004 -- tds
//

#ifndef VANK_INFERENCENETWORKBUILDER_HPP
#define VANK_INFERENCENETWORKBUILDER_HPP

#include "vank/QuerySpec.hpp"
#include "vank/Repository.hpp"
#include "vank/InferenceNetwork.hpp"
#include "vank/ListCache.hpp"
#include <map>
#include <vector>
namespace vank
{
  namespace infnet
  {

    class InferenceNetworkBuilder : public vank::lang::Walker {
    private:
      std::map< vank::lang::Node*, InferenceNetworkNode* > _nodeMap;
      InferenceNetwork* _network;
      vank::collection::Repository& _repository;
      vank::lang::ListCache& _cache;
      int _resultsRequested;
      int _maxWildcardTerms;

      template<typename _To, typename _From>
      std::vector<_To*> _translate( const std::vector<_From*>& children ) {
        std::vector<_To*> translation;

        for( unsigned int i=0; i<children.size(); i++ ) {
          translation.push_back( dynamic_cast<_To*>(_nodeMap[children[i]]) );
        }

        return translation;
      }

      vank::query::TermScoreFunction* _buildTermScoreFunction( const std::string& smoothing, double occurrences, double contextSize, int documentOccurrences, int documentCount ) const;

      void _after( vank::lang::NestedExtentInside* extentInside );
      void _after( vank::lang::NestedRawScorerNode* rawScorerNode );
      void _after( vank::lang::ShrinkageScorerNode* shrinkageScorerNode );
      void _after( vank::lang::ExtentEnforcement * eeNode );
      void _after( vank::lang::ExtentDescendant* extentInside );
      void _after( vank::lang::ExtentChild* extentInside );
      void _after( vank::lang::ExtentParent* extentInside );

    public:
      static const int DEFAULT_MAX_WILDCARD_TERMS = 100;

      InferenceNetworkBuilder( vank::collection::Repository& repository, vank::lang::ListCache& cache, int resultsRequested, int maxWildcardTerms=DEFAULT_MAX_WILDCARD_TERMS );
      ~InferenceNetworkBuilder();

      InferenceNetwork* getNetwork();

      void defaultAfter( vank::lang::Node* node );
      void after( vank::lang::IndexTerm* term );
      void after( vank::lang::Field* field );
      void after( vank::lang::ExtentAnd* extentAnd );
      void after( vank::lang::ExtentOr* extentOr );
      void after( vank::lang::WeightedExtentOr* weightedExtentOr );
      void after( vank::lang::ExtentInside* extentInside );
      void after( vank::lang::BAndNode* uwNode );
      void after( vank::lang::ODNode* odNode );
      void after( vank::lang::UWNode* uwNode );
      void after( vank::lang::FilRejNode* uwNode );
      void after( vank::lang::FilReqNode* uwNode );
      void after( vank::lang::ContextCounterNode* contextCounterNode );
      void after( vank::lang::ContextSimpleCounterNode* contextSimpleCounterNode );
      void after( vank::lang::ListAccumulator* listAccumulator );
      void after( vank::lang::ScoreAccumulatorNode* scoreAccumulatorNode );
      void after( vank::lang::AnnotatorNode* annotatorNode );
      void after( vank::lang::TermFrequencyScorerNode* termScorerNode );
      void after( vank::lang::CachedFrequencyScorerNode* cachedScorerNode );
      void after( vank::lang::RawScorerNode* rawScorerNode );
      void after( vank::lang::ExtentRestriction* erNode );
      void after( vank::lang::FixedPassage* fpNode );
      void after( vank::lang::PriorNode* priorNode );
      void after( vank::lang::WeightNode* weightNode );
      void after( vank::lang::CombineNode* combineNode );
      void after( vank::lang::PlusNode* plusNode );
      void after( vank::lang::WPlusNode* plusNode );
      void after( vank::lang::OrNode* orNode );
      void after( vank::lang::NotNode* notSpecNode );
      void after( vank::lang::MaxNode* maxNode );
      void after( vank::lang::WAndNode* wandNode );
      void after( vank::lang::WSumNode* wsumNode );
      void after( vank::lang::FieldLessNode* flNode );
      void after( vank::lang::FieldGreaterNode* fgNode );
      void after( vank::lang::FieldBetweenNode* fbNode );
      void after( vank::lang::FieldEqualsNode* fbNode );
      void after( vank::lang::FilterNode* filterNode );
      void after( vank::lang::FieldWildcard* fieldWildcard );
      void after( vank::lang::ContextInclusionNode* contextInclusionNode );
      void after( vank::lang::LengthPrior* lengthPrior );
      void after( vank::lang::DocumentStructureNode * docStruct );
      void after( vank::lang::ShrinkageScorerNode* shrinkageScorerNode );
      void after( vank::lang::ExtentDescendant* extentInside );
      void after( vank::lang::ExtentChild* extentInside );
      void after( vank::lang::ExtentParent* extentInside );
      void after( vank::lang::WildcardTerm* wildcardTerm );
    };
  }
}

#endif // VANK_INFERENCENETWORKBUILDER
