/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// ListBeliefNode
//
// 26 January 2004 - tds
//

#ifndef VANK_LISTBELIEFNODE_HPP
#define VANK_LISTBELIEFNODE_HPP

#include "vank/BeliefNode.hpp"
#include "vank/TermScoreFunction.hpp"
#include "vank/greedy_vector"
#include "vank/ListIteratorNode.hpp"
#include "vank/ScoredExtentResult.hpp"
namespace vank
{
  namespace infnet
  {

    class ListBeliefNode : public BeliefNode {
    private:
      ListIteratorNode& _list;
      ListIteratorNode* _context;
      ListIteratorNode* _raw;
      double _maximumScore;
      double _maximumBackgroundScore;
      vank::query::TermScoreFunction& _scoreFunction;
      vank::utility::greedy_vector<vank::api::ScoredExtentResult> _scores;
      vank::utility::greedy_vector<bool> _matches;
      std::string _name;
      bool _documentSmoothing;

    private:
      // computes the length of the scored context
      inline int _contextLength( vank::index::Extent &extent );
      inline double _contextOccurrences( vank::index::Extent &extent );
      inline double _documentOccurrences();

    public:
      ListBeliefNode( const std::string& name,
                      ListIteratorNode& child,
                      ListIteratorNode* context,
                      ListIteratorNode* raw,
                      vank::query::TermScoreFunction& scoreFunction,
                      double maximumBackgroundScore,
                      double maximumScore );

      lemur::api::DOCID_T nextCandidateDocument();
      void indexChanged( vank::index::Index& index );
      double maximumBackgroundScore();
      double maximumScore();
      const vank::utility::greedy_vector<vank::api::ScoredExtentResult>& score( lemur::api::DOCID_T documentID, vank::index::Extent &extent, int documentLength );
      void annotate( class Annotator& annotator, lemur::api::DOCID_T documentID, vank::index::Extent &extent );
      bool hasMatch( lemur::api::DOCID_T documentID );
      const vank::utility::greedy_vector<bool>& hasMatch( lemur::api::DOCID_T documentID, const vank::utility::greedy_vector<vank::index::Extent>& extents );
      const std::string& getName() const;
    };
  }
}

#endif // VANK_TERMNODE_HPP
