/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */

//
// ListCache
//
// 24 September 2004 -- tds
//
// Stores previously used precomputed lists.
//

#ifndef VANK_LISTCACHE_HPP
#define VANK_LISTCACHE_HPP

#include <vector>
#include "vank/delete_range.hpp"
#include "vank/SimpleCopier.hpp"
#include "vank/DocumentCount.hpp"
#include "vank/QuerySpec.hpp"
#include "vank/TreePrinterWalker.hpp"
#include "vank/Parameters.hpp"
namespace vank
{
  namespace lang
  {

    class ListCache {
    public:
      struct CachedList {
        // query structure
        vank::lang::SimpleCopier raw;
        vank::lang::SimpleCopier context;

        // postings
        vank::utility::greedy_vector<vank::index::DocumentContextCount> entries;

        // statistics about the entries
        INT64 occurrences;
        INT64 contextSize;
        INT64 minimumContextSize;
        INT64 maximumContextSize;
        INT64 maximumOccurrences;
        float maximumContextFraction;
      };

    private:
      std::vector<struct CachedList*> _lists;

    public:
      ~ListCache() {
        vank::utility::delete_vector_contents( _lists );
      }

      void add( CachedList* list ) {
        if( _lists.size() > 100 ) {
          delete _lists[0];
          _lists.erase( _lists.begin() );
        }

        _lists.push_back( list );
      }

      CachedList* find( vank::lang::Node* raw, vank::lang::Node* context ) {
        ListCache::CachedList* list = 0;
        size_t i = 0;

        // TODO: use a hash function to make this faster
        for( i=0; i<_lists.size(); i++ ) {
          vank::lang::Node* cachedRaw = _lists[i]->raw.root();
          vank::lang::Node* cachedContext = _lists[i]->context.root();

          if( *cachedRaw == *raw ) {
            if( ( !cachedContext && !context ) ||
                ( cachedContext && context && (*context == *cachedContext)) ) {
              list = _lists[i];
              break;
            }
          }
        }

        return list;
      }
    };
  }
}

#endif // VANK_LISTCACHE_HPP
