/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// NetworkServerStub
//
// 23 March 2004 -- tds
//

#ifndef VANK_NETWORKSERVERSTUB_HPP
#define VANK_NETWORKSERVERSTUB_HPP

#include "vank/XMLNode.hpp"
#include "vank/NetworkMessageStream.hpp"
#include "vank/QueryServer.hpp"
#include "vank/QueryResponsePacker.hpp"
namespace vank
{
  namespace net
  {

    class NetworkServerStub : public MessageStreamHandler {
    private:
      vank::server::QueryServer* _server;
      NetworkMessageStream* _stream;

      vank::xml::XMLNode* _encodeDocument( const struct vank::api::ParsedDocument* document );
      void _decodeMetadataRequest( const class vank::xml::XMLNode* request,
                                   std::string& attributeName,
                                   std::vector<std::string>& attributeValues );
      void _sendDocumentsResponse( class vank::server::QueryServerDocumentsResponse* response );
      void _sendNumericResponse( const char* responseName, UINT64 number );

      void _handleDocuments( vank::xml::XMLNode* input );
      void _handleDocumentMetadata( vank::xml::XMLNode* request );
      void _handleDocumentVectors( vank::xml::XMLNode* request );

      void _handleDocumentIDsFromMetadata( vank::xml::XMLNode* request );
      void _handleDocumentsFromMetadata( vank::xml::XMLNode* request );

      void _handleQuery( vank::xml::XMLNode* input );

      void _handleTermCount( vank::xml::XMLNode* request );
      void _handleTermCountUnique( vank::xml::XMLNode* request );
      void _handleStemCountText( vank::xml::XMLNode* request );
      void _handleTermCountText( vank::xml::XMLNode* request );

      void _handleTermName( vank::xml::XMLNode* request );
      void _handleTermID( vank::xml::XMLNode* request );
      void _handleStemTerm( vank::xml::XMLNode* request );

      void _handleTermFieldCount( vank::xml::XMLNode* request );
      void _handleStemFieldCount( vank::xml::XMLNode* request );
      void _handleDocumentCount( vank::xml::XMLNode* request );
      void _handleDocumentTermCount( vank::xml::XMLNode* request );
      void _handleDocumentStemCount( vank::xml::XMLNode* request );
      void _handleDocumentLength( vank::xml::XMLNode* request );
      void _handleFieldList( vank::xml::XMLNode* request );

      void _handlePathNames( vank::xml::XMLNode* request );

      void _handleSetMaxWildcardTerms( vank::xml::XMLNode* request );

    public:
      NetworkServerStub( vank::server::QueryServer* server, NetworkMessageStream* stream );
      void request( vank::xml::XMLNode* input );
      void reply( vank::xml::XMLNode* input );
      void reply( const std::string& name, const void* buffer, unsigned int length );
      void replyDone();
      void error( const std::string& error );
      void run();
    };
  }
}

#endif // VANK_NETWORKSERVERSTUB_HPP
