/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */

//
// NotNode
//
// 26 August 2004 -- tds
//

#ifndef VANK_NOTNODE_HPP
#define VANK_NOTNODE_HPP

#include "vank/BeliefNode.hpp"
#include "vank/greedy_vector"
#include <string>
namespace vank
{
  namespace infnet
  {

    class NotNode : public BeliefNode {
    private:
      BeliefNode* _child;
      vank::utility::greedy_vector<vank::api::ScoredExtentResult> _extents;
      vank::utility::greedy_vector<bool> _matches;
      std::string _name;

    public:
      NotNode( const std::string& name, BeliefNode* child );

      double maximumBackgroundScore();
      double maximumScore();
      const vank::utility::greedy_vector<vank::api::ScoredExtentResult>& score( lemur::api::DOCID_T documentID, vank::index::Extent &extent, int documentLength );
      bool hasMatch( lemur::api::DOCID_T documentID );
      const vank::utility::greedy_vector<bool>& hasMatch( lemur::api::DOCID_T documentID, const vank::utility::greedy_vector<vank::index::Extent>& extents );

      void annotate( class Annotator& annotator, lemur::api::DOCID_T documentID, vank::index::Extent &extent);

      lemur::api::DOCID_T nextCandidateDocument();
      void indexChanged( vank::index::Index& index );
      const std::string& getName() const;

      virtual void setSiblingsFlag(int f){
        bSiblings=f; // need to set the flag for the current node itself.
        if (_child) {  _child->setSiblingsFlag(f); }
      }

    };
  }
}

#endif // VANK_NOTNODE_HPP
