/*==========================================================================
 * Copyright (c) 2009 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// PlusNode
//
// 6 August 2009 - dmf
//

#ifndef VANK_PLUSNODE_HPP
#define VANK_PLUSNODE_HPP

#include <vector>
#include "vank/BeliefNode.hpp"
#include "vank/ScoredExtentResult.hpp"
#include <math.h>
namespace vank
{
  namespace infnet
  {

    class PlusNode : public BeliefNode {
    private:
      std::vector<BeliefNode*> _children;
      vank::utility::greedy_vector<vank::api::ScoredExtentResult> _scores;
      vank::utility::greedy_vector<bool> _matches;
      std::string _name;

    public:
      PlusNode( const std::string& name, const std::vector<BeliefNode*>& children  ) : _name(name), _children(children) {}

      // InferenceNetworkNode interface
      lemur::api::DOCID_T nextCandidateDocument();
      void indexChanged( vank::index::Index& index );
      double maximumScore();
      double maximumBackgroundScore();
      vank::utility::greedy_vector<vank::api::ScoredExtentResult>& score( lemur::api::DOCID_T documentID, vank::index::Extent &extent, int documentLength );
      void annotate( class Annotator& annotator, lemur::api::DOCID_T documentID, vank::index::Extent &extent );
      bool hasMatch( lemur::api::DOCID_T documentID );
      const vank::utility::greedy_vector<bool>& hasMatch( lemur::api::DOCID_T documentID, const vank::utility::greedy_vector<vank::index::Extent>& extents );
      const std::string& getName() const;
    };
  }
}

#endif // VANK_PLUSNODE_HPP
