/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */

//
// PriorNode
//
// 27 April 2004 -- tds
//

#ifndef VANK_PRIORNODE_HPP
#define VANK_PRIORNODE_HPP

#include "vank/BeliefNode.hpp"
#include "vank/QuerySpec.hpp"
#include "vank/PriorListIterator.hpp"

namespace vank
{
  namespace infnet
  {
    class PriorNode : public BeliefNode {
    private:
      vank::utility::greedy_vector<bool> _matches;
      vank::utility::greedy_vector<vank::api::ScoredExtentResult> _scores;
      std::string _name;

      class InferenceNetwork& _network;
      int _listID;

      vank::collection::PriorListIterator* _iterator;

    public:
      PriorNode( const std::string& name,
                 class InferenceNetwork& network,
                 int listID );
      ~PriorNode();

      lemur::api::DOCID_T nextCandidateDocument();
      void indexChanged( vank::index::Index& index );

      bool hasMatch( lemur::api::DOCID_T documentID );
      const vank::utility::greedy_vector<bool>& hasMatch( lemur::api::DOCID_T documentID, const vank::utility::greedy_vector<vank::index::Extent>& extents );
      const vank::utility::greedy_vector<vank::api::ScoredExtentResult>& score( lemur::api::DOCID_T documentID, vank::index::Extent &extent, int documentLength );
      void annotate( class Annotator& annotator, lemur::api::DOCID_T documentID, vank::index::Extent &extent );
      double maximumScore();
      double maximumBackgroundScore();
      const std::string& getName() const;
    };
  }
}

#endif // VANK_PRIORNODE_HPP
