/*==========================================================================
 * Copyright (c) 2009 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// QueryTFWalker
//
// 07 August 2009 -- dmf
//

#ifndef VANK_QUERYTFWALKER_HPP
#define VANK_QUERYTFWALKER_HPP

#include <string>
#include <map>
#include <vector>
#include <iostream>

#include "vank/QuerySpec.hpp"
#include "vank/Walker.hpp"
#include "vank/QueryServer.hpp"

namespace vank
{
  namespace lang
  {

    class QueryTFWalker : public vank::lang::Walker {
    private:
      // map query term -> RawScorerNode list
      // qTF is the length of the list
      std::map< lemur::api::TERMID_T, std::vector<vank::lang::RawScorerNode*> > _nodeMap;
      std::vector<vank::server::QueryServer*> &_servers;

    public:
      QueryTFWalker( std::vector<vank::server::QueryServer*> & servers ) : _servers(servers) {
      }

      ~QueryTFWalker( ) {
      }

      void after( vank::lang::RawScorerNode* scorer ) {
        // have to stem the term to get the counts right.
        lemur::api::TERMID_T id = 0;
        // take the first id returned
        for (int i = 0; (id == 0) && (i < _servers.size()); i++)
          id = _servers[i]->termID(scorer->queryText());
        _nodeMap[id].push_back(scorer);
      }
      void after( vank::lang::PlusNode *plus) {
        std::map< lemur::api::TERMID_T, std::vector<vank::lang::RawScorerNode*> >::iterator iter;
        for (iter = _nodeMap.begin(); iter != _nodeMap.end(); iter++) {
          std::vector<vank::lang::RawScorerNode *> & nodes = iter->second;
          for (int i = 0; i < nodes.size(); i++) {
            std::string smoothing = nodes[i]->getSmoothing();
            int qTF = nodes.size();
            std::stringstream smooth;
            smooth << smoothing << ",qtf:" << qTF;
            nodes[i]->setSmoothing( smooth.str() );
          }
        }
      }
      void after( vank::lang::WPlusNode *plus) {
        const std::vector< std::pair<double, ScoredExtentNode*> >& children = plus->getChildren();
        for( size_t i=0; i<children.size(); i++ ) {
          double weight = children[i].first;
          vank::lang::RawScorerNode * raw = dynamic_cast<vank::lang::RawScorerNode *>( children[i].second );
          std::string smoothing = raw->getSmoothing();
          std::stringstream smooth;
          smooth << smoothing << ",qtw:" << weight;
          raw->setSmoothing( smooth.str() );
        }
      }
    };
  }
}

#endif // VANK_QUERYTFWALKER_HPP
