/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// RawScorerNodeExtractor.hpp
//
// 5 March 2004 -- tds
//

#ifndef VANK_RAWSCORERNODEEXTRACTOR_HPP
#define VANK_RAWSCORERNODEEXTRACTOR_HPP

#include "vank/Walker.hpp"
#include "vank/QuerySpec.hpp"
namespace vank
{
  namespace lang
  {

    class RawScorerNodeExtractor : public vank::lang::Walker {
    private:
      std::vector<vank::lang::RawScorerNode*> _scorerNodes;
      std::vector<vank::lang::Node*> _reachableNodes;
      int _insideCount; // used unitialized

    public:
      RawScorerNodeExtractor() : _insideCount(0) { }

      void defaultBefore( vank::lang::Node* n ) {
        if( _insideCount ) {
          _reachableNodes.push_back(n);
        }
      }

      void before( vank::lang::RawScorerNode* node ) {
        _insideCount++;
        _scorerNodes.push_back(node);
        _reachableNodes.push_back(node);
      }

      void after( vank::lang::RawScorerNode* node ) {
        _insideCount--;
      }

      void before( vank::lang::NestedRawScorerNode* node ) {
        before( (vank::lang::RawScorerNode *) node );
      }

      void after( vank::lang::NestedRawScorerNode* node ) {
        after( (vank::lang::RawScorerNode *) node );
      }

      void before( vank::lang::ShrinkageScorerNode* node ) {
        before( (vank::lang::RawScorerNode *) node );
      }

      void after( vank::lang::ShrinkageScorerNode* node ) {
        after( (vank::lang::RawScorerNode *) node );
      }

      std::vector<vank::lang::RawScorerNode*>& getScorerNodes() {
        return _scorerNodes;
      }

      std::vector<vank::lang::Node*>& getReachableNodes() {
        return _reachableNodes;
      }
    };
  }
}

#endif // VANK_RAWSCORERNODEEXTRACTOR_HPP
