/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// ShrinkageBeliefNode
//
// 29 June 2005 - pto
//

#ifndef VANK_SHRINKAGEBELIEFNODE_HPP
#define VANK_SHRINKAGEBELIEFNODE_HPP

#include "vank/BeliefNode.hpp"
#include "vank/TermScoreFunction.hpp"
#include "vank/greedy_vector"
#include "vank/ScoredExtentResult.hpp"
#include "vank/DocumentStructureHolderNode.hpp"
#include <vector>
#include <set>
#include <map>

namespace vank
{
  namespace infnet
  {

    class ShrinkageBeliefNode : public BeliefNode {
    protected:
      ListIteratorNode& _list;
      DocumentStructureHolderNode& _docStructHolder;
      double _maximumScore;
      double _maximumBackgroundScore;
      vank::query::TermScoreFunction& _scoreFunction;
      vank::utility::greedy_vector<vank::api::ScoredExtentResult> _results;
      vank::utility::greedy_vector<bool> _matches;
      std::string _name;

      vank::utility::greedy_vector<double> _down;
      vank::utility::greedy_vector<double> _up;
      vank::utility::greedy_vector<double> _base;
      vank::utility::greedy_vector<double> _counts;
      lemur::api::DOCID_T _documentID;


      typedef struct sr {
        std::string fieldName;
        double weight;
        bool lengthProportional;
      } smoothing_rule;

      struct lt_rule {
        bool operator()( smoothing_rule r1, smoothing_rule r2 ) const {
          int cmpVal = strcmp( r1.fieldName.c_str(), r2.fieldName.c_str() );
          if ( cmpVal != 0 ) {
            return cmpVal < 0;
          }
          if ( r1.weight != r2.weight ) {
            return r1.weight < r2.weight;
          }
          if ( r1.lengthProportional != r2.lengthProportional) {
            return r1.lengthProportional;
          }
          return false;
        }
      };

      std::set<smoothing_rule, lt_rule> _ruleSet;
      std::map<int, smoothing_rule> _ruleMap;

      double _parentWeight;
      double _docWeight;
      double _otherWeight;
      bool _recursive;
      bool _queryLevelCombine;
      double _defaultScore;


      std::set<int> _roots;
      vank::utility::greedy_vector<int> _topDownOrder;

      void _buildScoreCache( lemur::api::DOCID_T documentID );

    public:
      ShrinkageBeliefNode( const std::string& name,
                           ListIteratorNode& child,
                           DocumentStructureHolderNode& documentStructureHolderNode,
                           vank::query::TermScoreFunction& scoreFunction,
                           double maximumBackgroundScore,
                           double maximumScore );

      lemur::api::DOCID_T nextCandidateDocument();
      void indexChanged( vank::index::Index& index );
      double maximumBackgroundScore();
      double maximumScore();
      const vank::utility::greedy_vector<vank::api::ScoredExtentResult>& score( lemur::api::DOCID_T documentID, vank::index::Extent &extent, int documentLength );
      void annotate( class Annotator& annotator, lemur::api::DOCID_T documentID, vank::index::Extent &extent );
      bool hasMatch( lemur::api::DOCID_T documentID );
      const vank::utility::greedy_vector<bool>& hasMatch( lemur::api::DOCID_T documentID, const vank::utility::greedy_vector<vank::index::Extent>& extents );
      const std::string& getName() const;
      void addShrinkageRule( std::string rule );
      // so that we can know collection/document lambdas of linear interpolation
      void setSmoothing( const std::string & stringSpec );
    };
  }
}

#endif // VANK_SHRINKAGEBELIEFNODE_HPP
