

//
// SnippetBuilder
//
// This code is based largely on the code I wrote for the PHP and Java
// interfaces.
//
// 17 July 2006 -- tds
//

#ifndef VANK_SNIPPETBUILDER_HPP
#define VANK_SNIPPETBUILDER_HPP

#include <vector>
#include <string>
#include "vank/QueryAnnotation.hpp"
#include "vank/ParsedDocument.hpp"

namespace vank {
  namespace api {
    class SnippetBuilder {
    private:
      bool _HTMLOutput;

    public:
      struct Region {
        int begin;
        int end;

        std::vector<vank::index::Extent> extents;
      };

    private:
      void _getRawNodes( std::vector<std::string>& nodeNames, const vank::api::QueryAnnotationNode* node );

      std::vector< std::pair<vank::index::Extent, int> > _documentMatches( int document,
                                                                            const std::map< std::string, std::vector<vank::api::ScoredExtentResult> >& annotations,
                                                                            const std::vector<std::string>& nodeNames );

      std::vector<Region> _buildRegions(
                                        std::vector< std::pair<vank::index::Extent, int> >& extents,
                                        int positionCount, int matchWidth, int windowWidth );

      Region _bestRegion( const std::vector< std::pair<vank::index::Extent, int> >& extents,
                          const std::vector< vank::api::SnippetBuilder::Region >& skipRegions,
                          int positionCount, int windowWidth );

      char* _sanitizeText( const char* text, int begin, int length );

      void _addEllipsis( std::string& snippet );
      void _addHighlightedRegion( std::string& snippet, char* region );
      void _addUnhighlightedRegion( std::string& snippet, char* region );
      void _completeSnippet( std::string& snippet );

    public:
      SnippetBuilder( bool html ) : _HTMLOutput(html) {}
      std::string build( int documentID, const vank::api::ParsedDocument* document, vank::api::QueryAnnotation* annotation );
      std::vector< std::pair< std::string, std::string > >* buildField( int documentID, const vank::api::ParsedDocument* document, vank::api::QueryAnnotation* annotation, std::vector< std::string > fds );
    };
  }
}

#endif // VANK_SNIPPETBUILDER_HPP
