/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */

//
// TermFrequencyBeliefNode
//
// 25 August 2004 -- tds
//

#ifndef VANK_TERMFREQUENCYBELIEFNODE_HPP
#define VANK_TERMFREQUENCYBELIEFNODE_HPP

#include "vank/Annotator.hpp"
#include "vank/ScoredExtentResult.hpp"
#include <string>
#include "vank/TermScoreFunction.hpp"
#include "vank/ListBeliefNode.hpp"
#include "vank/DocListIterator.hpp"
namespace vank
{
  namespace infnet
  {

    class TermFrequencyBeliefNode : public BeliefNode {
    private:
      class InferenceNetwork& _network;
      vank::query::TermScoreFunction& _function;
      vank::utility::greedy_vector<vank::api::ScoredExtentResult> _extents;
      vank::utility::greedy_vector<bool> _matches;
      vank::index::DocListIterator* _list;
      double _maximumBackgroundScore;
      double _maximumScore;
      std::string _name;
      int _listID;

      vank::utility::greedy_vector<vank::index::DocListIterator::TopDocument> _emptyTopdocs;

    public:
      TermFrequencyBeliefNode( const std::string& name,
                               class InferenceNetwork& network,
                               int listID,
                               vank::query::TermScoreFunction& scoreFunction );

      ~TermFrequencyBeliefNode();

      const vank::utility::greedy_vector<vank::index::DocListIterator::TopDocument>& topdocs() const;
      lemur::api::DOCID_T nextCandidateDocument();
      void indexChanged( vank::index::Index& index );
      double maximumBackgroundScore();
      double maximumScore();
      const vank::utility::greedy_vector<vank::api::ScoredExtentResult>& score( lemur::api::DOCID_T documentID, vank::index::Extent &extent, int documentLength );
      void annotate( class Annotator& annotator, lemur::api::DOCID_T documentID, vank::index::Extent &extent );
      bool hasMatch( lemur::api::DOCID_T documentID );
      const vank::utility::greedy_vector<bool>& hasMatch( lemur::api::DOCID_T documentID, const vank::utility::greedy_vector<vank::index::Extent>& extents );
      const std::string& getName() const;
    };
  }
}

#endif // VANK_TERMFREQUENCYBELIEFNODE_HPP
