/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// UnnecessaryNodeRemoverCopier
//
// 17 March 2004 -- tds
//

#ifndef VANK_UNNECESSARYNODEREMOVERCOPIER_HPP
#define VANK_UNNECESSARYNODEREMOVERCOPIER_HPP

#include "vank/QuerySpec.hpp"
#include "vank/delete_range.hpp"
namespace vank
{
  namespace lang
  {

    class UnnecessaryNodeRemoverCopier : public vank::lang::Copier {
    protected:
      std::vector<vank::lang::Node*> _nodes;

      class SingleChildWalker : public vank::lang::Walker {
      private:
        vank::lang::Node* _child;
        int _children;
        bool _seenRoot;

      public:
        SingleChildWalker() : _seenRoot(false), _children(0), _child(0) {}

        void defaultBefore( vank::lang::Node* n ) {
          if( !_seenRoot ) {
            _seenRoot = true;
          } else {
            _children++;
            _child = n;
          }
        }

        bool onlyOneChild() {
          return _children == 1;
        }

        vank::lang::Node* getChild() {
          return _child;
        }
      };

    public:
      ~UnnecessaryNodeRemoverCopier() {
        vank::utility::delete_vector_contents( _nodes );
      }

      vank::lang::Node* defaultAfter( vank::lang::Node* old, vank::lang::Node* newNode ) {
        _nodes.push_back( newNode );
        return newNode;
      }

      vank::lang::Node* after( vank::lang::ExtentAnd* oldAnd, vank::lang::ExtentAnd* newAnd ) {
        SingleChildWalker walker;
        newAnd->walk(walker);

        if( walker.onlyOneChild() ) {
          delete newAnd;
          return walker.getChild();
        } else {
          _nodes.push_back( newAnd );
          return newAnd;
        }
      }

      vank::lang::Node* after( vank::lang::ExtentOr* oldOr, vank::lang::ExtentOr* newOr ) {
        SingleChildWalker walker;
        newOr->walk(walker);

        if( walker.onlyOneChild() ) {
          delete newOr;
          return walker.getChild();
        } else {
          _nodes.push_back( newOr );
          return newOr;
        }
      }

      vank::lang::Node* after( vank::lang::ODNode* oldOD, vank::lang::ODNode* newOD ) {
        SingleChildWalker walker;
        newOD->walk(walker);

        if( walker.onlyOneChild() ) {
          delete newOD;
          return walker.getChild();
        } else {
          _nodes.push_back( newOD );
          return newOD;
        }
      }

      vank::lang::Node* after( vank::lang::UWNode* oldUW, vank::lang::UWNode* newUW ) {
        SingleChildWalker walker;
        newUW->walk(walker);

        if( walker.onlyOneChild() ) {
          delete newUW;
          return walker.getChild();
        } else {
          _nodes.push_back( newUW );
          return newUW;
        }
      }

      vank::lang::Node* after( vank::lang::LengthPrior * oldLP, vank::lang::LengthPrior * newLP ) {
        if ( oldLP->getExponent() == 0 ) {
          vank::lang::Node* child = newLP->getChild();
          delete newLP;
          return child;
        } else {
          _nodes.push_back( newLP );
          return newLP;
        }
      }

    };
  }
}

#endif // VANK_UNNECESSARYNODEREMOVERCOPIER_HPP
