/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// UnorderedWindowNode
//
// 26 January 2004 -- tds
// 11 December 2007 -- mjh - added child sort
//

#ifndef VANK_UNORDEREDWINDOWNODE_HPP
#define VANK_UNORDEREDWINDOWNODE_HPP

#include "vank/ListIteratorNode.hpp"
#include <vector>
#include <vank/greedy_vector>
namespace vank
{
  namespace infnet
  {

    // comparison operation used in child sorting
    // for ascending order by # of extents in each child
    inline bool UWNodeChildLess(vank::infnet::ListIteratorNode* _child1, vank::infnet::ListIteratorNode* _child2) {
      const vank::utility::greedy_vector<vank::index::Extent>& childPositions1 = _child1->extents();
      const vank::utility::greedy_vector<vank::index::Extent>& childPositions2 = _child2->extents();
      return (childPositions1.size() < childPositions2.size());
    }

    class UnorderedWindowNode : public ListIteratorNode {
    private:
      struct term_position {
        bool operator< ( const term_position& other ) const {
          return begin < other.begin;
        }

        double weight;
        int type;
        int begin;
        int end;
        int last; // index of previous entry of this type
      };

      int _windowSize;
      std::vector<ListIteratorNode*> _children;
      vank::utility::greedy_vector<vank::index::Extent> _extents;
      std::string _name;
      bool _childrenAlreadySorted;

    public:
      UnorderedWindowNode( const std::string& name, std::vector<ListIteratorNode*>& children );
      UnorderedWindowNode( const std::string& name, std::vector<ListIteratorNode*>& children, int windowSize );
      lemur::api::DOCID_T nextCandidateDocument();
      void indexChanged( vank::index::Index& index );
      void prepare( lemur::api::DOCID_T documentID );
      const vank::utility::greedy_vector<vank::index::Extent>& extents();
      const std::string& getName() const;
      void annotate( Annotator& annotator, lemur::api::DOCID_T documentID, vank::index::Extent &extent );
    };
  }
}

#endif // VANK_UNORDEREDWINDOWNODE_HPP
