#ifndef _VANKSEARCH_INTERFACE_H
#define _VANKSEARCH_INTERFACE_H

#include <string>
using std::string;

#include "CGIOutput.h"
#include "vank/QueryEnvironment.hpp"
#include "vank/QueryAnnotation.hpp"
#include "vank/ScoredExtentResult.hpp"
#include "vank/SnippetBuilder.hpp"

#define MIN(a,b) (((a) < (b)) ? (a) : (b))

struct sortScoredExtentByPosition {
  bool operator()(const vank::api::ScoredExtentResult &first, const vank::api::ScoredExtentResult &second) {
    return (first.begin < second.begin);
  }
};

/**
 * A separte class for searches via Vank (to clean things up a bit...
 *
 * @author Mark J. Hoy [http://www.cs.cmu.edu/~mhoy/]
 * @version 6/29/06
 */
class VankSearchInterface {
private:

  struct internalPos {
    int begin;
    int end;
  };

  struct vankTermMatches {
    int begin;
    int end;
    std::vector<internalPos> internalPositions;

    vankTermMatches(int b, int e) {
      begin=b;
      end=e;
    }
    ~vankTermMatches() {
      internalPositions.clear();
    }
  };

  CGIOutput *output;
  vank::api::QueryEnvironment *queryEnvironment;
  string dataRoot;

  int findOccurrence(char *input, char *chrSet);

  string replaceAllByChr(string inputString, string characters);

  void findAndReplace(std::string &source, const std::string &find, const std::string &replace);

  string stripHtmlTags(string inputString);

  string escapeHtmlTags(string inputString);

  std::string getASCIIFromPercentEncoding(std::string inputSequence);

  std::string normalizeURL(std::string inputURL);

  void addToStringVector(std::vector<std::string> *origVec, std::vector<std::string> *addVec);

  std::vector<std::string> getRawNodes(vank::api::QueryAnnotationNode *node);

  std::vector<VankSearchInterface::vankTermMatches> getMatches(int docID, std::map<std::string, std::vector<vank::api::ScoredExtentResult> > *annotations, std::vector<string> *rawNodes);

  std::vector<vank::api::ScoredExtentResult> vankRemoveDuplicateResults(std::vector<vank::api::ScoredExtentResult> results, vank::api::QueryEnvironment *vankEnvironment);

  std::string getScoredExtentSummaryString(vank::api::ScoredExtentResult &result, std::vector<string> *nodes,
                                           std::map< std::string, std::vector<vank::api::ScoredExtentResult> > *annotations,
                                           string docext);

  std::vector<std::string> getRawScoringNodes(const vank::api::QueryAnnotationNode *node);

  /**
   * Reformulates a free-text query
   * @param origQuery the original query
   * @return the (possibly) modified query string
   */
  string vankDefaultQueryExpansion(string &origQuery, bool usePagerank=false);

public:
  VankSearchInterface(CGIOutput *_output, vank::api::QueryEnvironment *_queryEnvironment, string _dataRoot);
  ~VankSearchInterface();
  void performSearch(string &query, int maxNumResults, int indexID, int listLength, int rankStart);



}; // class VankSearchInterface

#endif // _VANKSEARCH_INTERFACE_H
