/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// Annotator
//
// 26 July 2004 -- tds
//

#include "vank/Annotator.hpp"

vank::infnet::Annotator::Annotator( const std::string& name, BeliefNode* belief )
  :
  _name(name),
  _belief(belief)
{
}

void vank::infnet::Annotator::add( InferenceNetworkNode* node, lemur::api::DOCID_T documentID, vank::index::Extent &extent ) {
  vank::api::ScoredExtentResult a(extent);

  a.document = documentID;
  a.score = 0;

  std::set<vank::api::ScoredExtentResult, vank::api::ScoredExtentResult::score_greater> &nodeSeen = _seen[node->getName()];

  if ( nodeSeen.find( a ) != nodeSeen.end() )
    return;

  nodeSeen.insert(a);
  _annotations[node->getName()].push_back(a);
}

void vank::infnet::Annotator::addMatches( vank::utility::greedy_vector<vank::index::Extent>& extents, InferenceNetworkNode* node, lemur::api::DOCID_T documentID, vank::index::Extent &extent ) {
  vank::index::Extent range( extent.begin, extent.end );

  vank::utility::greedy_vector<vank::index::Extent>::const_iterator iter;
  iter = std::lower_bound( extents.begin(), extents.end(), range, vank::index::Extent::begins_before_less() );

  for( size_t i = iter - extents.begin(); i<extents.size(); i++ ) {
    if( extent.begin > extents[i].begin )
      continue;

    if( extent.end < extents[i].end )
      continue;

    add( node, documentID, (vank::index::Extent &)extents[i]);
  }
}

void vank::infnet::Annotator::evaluate( lemur::api::DOCID_T documentID, int documentLength ) {
  vank::index::Extent tmpExtent(0, documentLength);
  _belief->annotate( *this, documentID, tmpExtent );
}

lemur::api::DOCID_T vank::infnet::Annotator::nextCandidateDocument() {
  return _belief->nextCandidateDocument();
}

vank::infnet::EvaluatorNode::MResults& vank::infnet::Annotator::getResults() {
  return _annotations;
}

const std::string& vank::infnet::Annotator::getName() const {
  return _name;
}

const vank::infnet::EvaluatorNode::MResults& vank::infnet::Annotator::getResults() const {
  return _annotations;
}

void vank::infnet::Annotator::indexChanged( vank::index::Index& index ) {
  // do nothing
}
