/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */

//
// DiskDocListFileIterator
//
// 13 December 2004 -- tds
//

#include "vank/DiskDocListFileIterator.hpp"

//
// _readEntry
//

void vank::index::DiskDocListFileIterator::_readEntry() {
  int headerLength;
  UINT64 startPosition;

  // figure out where we are, and how much header data needs to be read
  startPosition = _file->position();
  _file->read( &headerLength, sizeof headerLength );

  // read in the header
  _header.clear();

  _file->read( _header.write( headerLength ), headerLength );
  vank::utility::RVLDecompressStream stream( _header.front(), _header.position() );

  // header is RVLCompressed with the term first, followed by a termData structure
  stream >> _term;
  ::termdata_decompress( stream, _termData, _fieldCount );
  _termData->term = _term;

  // set up the iterator to find the appropriate data
  _iterator.setStartOffset( startPosition, _termData );
  _iterator.startIteration();

  _docListData.iterator = &_iterator;
  _docListData.termData = _termData;
}

//
// DiskDocListFileIterator
//

vank::index::DiskDocListFileIterator::DiskDocListFileIterator( vank::file::File& docListFile, int fieldCount ) :
  _file( new vank::file::SequentialReadBuffer( docListFile ) ),
  _fileLength( docListFile.size() ),
  _fieldCount( fieldCount ),
  _iterator( _file, 0, 0 ),
  _finished( false )
{
  _termData = (vank::index::TermData*) malloc( ::termdata_size( fieldCount ) );
}

//
// DiskDocListFileIterator destructor
//

vank::index::DiskDocListFileIterator::~DiskDocListFileIterator() {
  free( _termData );
}

//
// startIteration
//

void vank::index::DiskDocListFileIterator::startIteration() {
  _finished = false;
  _readEntry();
}

//
// nextEntry
//

bool vank::index::DiskDocListFileIterator::nextEntry() {
  if( _file->position() < _fileLength ) {
    _readEntry();
    return true;
  }

  _finished = true;
  return false;
}

//
// currentEntry
//

vank::index::DocListFileIterator::DocListData* vank::index::DiskDocListFileIterator::currentEntry() {
  return &_docListData;
}

//
// currentEntry
//

const vank::index::DocListFileIterator::DocListData* vank::index::DiskDocListFileIterator::currentEntry() const {
  return &_docListData;
}

//
// finished
//

bool vank::index::DiskDocListFileIterator::finished() const {
  return _finished;
}
