/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */

//
// DiskDocumentDataIterator
//
// 21 January 2004 -- tds
//

#include "vank/DiskDocumentDataIterator.hpp"

//
// DiskDocumentDataIterator constructor
//

vank::index::DiskDocumentDataIterator::DiskDocumentDataIterator( vank::file::File& documentDataFile ) :
  _documentDataFile(documentDataFile),
  _readBuffer( new vank::file::SequentialReadBuffer( documentDataFile, 1024*1024 ) ),
  _finished(false)
{
}

//
// DiskDocumentDataIterator destructor
//

vank::index::DiskDocumentDataIterator::~DiskDocumentDataIterator() {
  delete _readBuffer;
}

//
// startIteration
//

void vank::index::DiskDocumentDataIterator::startIteration() {
  _fileSize = _documentDataFile.size();
  _readBuffer->seek( 0 );
  if( _fileSize ) {
    _readBuffer->read( &_documentData, sizeof(DocumentData) );
    _finished = false;
  } else {
    _finished = true;
  }
}

//
// nextEntry
//

bool vank::index::DiskDocumentDataIterator::nextEntry() {
  if( _readBuffer->position() < _fileSize ) {
    _readBuffer->read( &_documentData, sizeof(DocumentData) );
    return true;
  }

  _finished = true;
  return false;
}

//
// currentEntry
//

const vank::index::DocumentData* vank::index::DiskDocumentDataIterator::currentEntry() {
  if( !finished() )
    return &_documentData;

  return 0;
}

//
// finished
//

bool vank::index::DiskDocumentDataIterator::finished() {
  return _finished;
}
