/*==========================================================================
 * Copyright (c) 2006 Carnegie Mellon University.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software (and below), and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
*/
//
// LengthPriorNode
//
// 7 July 2005 -- pto
//

#include "vank/LengthPriorNode.hpp"
#include <math.h>
#include "vank/Annotator.hpp"

vank::infnet::LengthPriorNode::LengthPriorNode( const std::string& name, BeliefNode * child, double exponent ) :
  _exponent(exponent),
  _name(name),
  _child(child)
{
}

vank::infnet::LengthPriorNode::~LengthPriorNode() {
}

lemur::api::DOCID_T vank::infnet::LengthPriorNode::nextCandidateDocument() {
  return _child->nextCandidateDocument();
}

bool vank::infnet::LengthPriorNode::hasMatch( lemur::api::DOCID_T documentID ) {
  return _child->hasMatch( documentID );
}

const vank::utility::greedy_vector<bool>& vank::infnet::LengthPriorNode::hasMatch( lemur::api::DOCID_T documentID, const vank::utility::greedy_vector<vank::index::Extent>& extents ) {
  return _child->hasMatch( documentID, extents );
}

const vank::utility::greedy_vector<vank::api::ScoredExtentResult>& vank::infnet::LengthPriorNode::score( lemur::api::DOCID_T documentID, vank::index::Extent &extent, int documentLength ) {
  _scores.clear();
  const vank::utility::greedy_vector<vank::api::ScoredExtentResult>& childResults = _child->score( documentID, extent, documentLength );
  for( size_t j=0; j<childResults.size(); j++ ) {
    vank::api::ScoredExtentResult result(childResults[j]);

    double score = childResults[j].score ;
    int b = childResults[j].begin;
    int e = childResults[j].end;
    double length = e - b;

    if ( length > 0 ) {
      score += _exponent * log( length );
    } else {
      score = VANK_TINY_SCORE;
    }

    result.score=score;
    result.document=documentID;
    result.begin=b;
    result.end=e;

    _scores.push_back( result );
  }
  return _scores;
}

void vank::infnet::LengthPriorNode::annotate( class vank::infnet::Annotator& annotator, lemur::api::DOCID_T documentID, vank::index::Extent &extent ) {
  annotator.add( this, documentID, extent);
  _child->annotate( annotator, documentID, extent );

}

double vank::infnet::LengthPriorNode::maximumScore() {
  return VANK_HUGE_SCORE;
}

double vank::infnet::LengthPriorNode::maximumBackgroundScore() {
  return VANK_TINY_SCORE;
}

const std::string& vank::infnet::LengthPriorNode::getName() const {
  return _name;
}

void vank::infnet::LengthPriorNode::indexChanged( vank::index::Index& index ) {
}
