/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */

//
// ListAccumulator
//
// 11 April 2006 -- tds
//
// This class exists only for the QueryEnvironment::expressionList
// call.  Its job is to collect the occurrences of a particular
// query language expression, and to return those occurrences to
// the caller.
//

#include "vank/ListAccumulator.hpp"
#include "vank/ScoredExtentResult.hpp"

vank::infnet::ListAccumulator::ListAccumulator( const std::string& nodeName,
                                                 vank::infnet::ListIteratorNode& iterNode ) :
  _name(nodeName),
  _counted(iterNode)
{
  _results[ "occurrences" ].clear();
  _resultVector = &_results[ "occurrences" ];
}

vank::infnet::ListAccumulator::~ListAccumulator() {
}

const std::string& vank::infnet::ListAccumulator::getName() const {
  return _name;
}

const vank::infnet::EvaluatorNode::MResults& vank::infnet::ListAccumulator::getResults() {
  return _results;
}

void vank::infnet::ListAccumulator::indexChanged( vank::index::Index& index ) {
}

void vank::infnet::ListAccumulator::evaluate( lemur::api::DOCID_T documentID, int documentLength ) {
  const vank::utility::greedy_vector<vank::index::Extent>& extents = _counted.extents();

  for( size_t i=0; i<extents.size(); i++ ) {
    vank::api::ScoredExtentResult result(extents[i]);
    result.document=documentID;
    _resultVector->push_back( result );
  }
}

lemur::api::DOCID_T vank::infnet::ListAccumulator::nextCandidateDocument() {
  return _counted.nextCandidateDocument();
}
