/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// NullScorerNode
//
// 6 July 2004 -- tds
//

#include "vank/NullScorerNode.hpp"
#include "lemur/lemur-compat.hpp"

vank::infnet::NullScorerNode::NullScorerNode( const std::string& name, vank::query::TermScoreFunction& scoreFunction ) :
  _name(name),
  _scoreFunction(scoreFunction),
  _maximumBackgroundScore(0),
  _maximumScore(0)
{
}

lemur::api::DOCID_T vank::infnet::NullScorerNode::nextCandidateDocument() {
  return MAX_INT32;
}

double vank::infnet::NullScorerNode::maximumScore() {
  return _maximumScore;
}

double vank::infnet::NullScorerNode::maximumBackgroundScore() {
  return _maximumBackgroundScore;
}

bool vank::infnet::NullScorerNode::hasMatch( lemur::api::DOCID_T documentID ) {
  return false;
}

const vank::utility::greedy_vector<bool>& vank::infnet::NullScorerNode::hasMatch( lemur::api::DOCID_T documentID, const vank::utility::greedy_vector<vank::index::Extent>& extents ) {
  return _matches;
}

const vank::utility::greedy_vector<vank::api::ScoredExtentResult>& vank::infnet::NullScorerNode::score( lemur::api::DOCID_T documentID, vank::index::Extent &extent, int documentLength ) {
  _scores.clear();
  double score = _scoreFunction.scoreOccurrence(0, documentLength);
  vank::api::ScoredExtentResult result(extent);
  result.score=score;
  result.document=documentID;
  _scores.push_back( result );

  return _scores;
}

void vank::infnet::NullScorerNode::annotate( vank::infnet::Annotator& annotator, lemur::api::DOCID_T documentID, vank::index::Extent &extent ) {
  // no need to annotate; there will never be any matches
}

const std::string& vank::infnet::NullScorerNode::getName() const {
  return _name;
}

void vank::infnet::NullScorerNode::indexChanged( vank::index::Index& index ) {
  _maximumBackgroundScore = VANK_HUGE_SCORE;
  _maximumScore = VANK_HUGE_SCORE;
}
