/*==========================================================================
 * Copyright (c) 2009 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */



//
// PlusNode
//
// 06 August 2009 - dmf
//

#include "vank/PlusNode.hpp"
#include <algorithm>
#include "lemur/lemur-platform.h"
#include <iostream>
#include "vank/Annotator.hpp"
#include "vank/TermFrequencyBeliefNode.hpp"
#include "vank/greedy_vector"
#include "vank/delete_range.hpp"
#include "vank/Parameters.hpp"
#include "vank/ExtentRestrictionNode.hpp"


void vank::infnet::PlusNode::indexChanged( vank::index::Index& index ) {
  // do nothing
}

lemur::api::DOCID_T vank::infnet::PlusNode::nextCandidateDocument() {
  lemur::api::DOCID_T candidate = MAX_INT32;
  for( unsigned int i=0; i<_children.size(); i++ ) {
    candidate = lemur_compat::min<lemur::api::DOCID_T>( candidate, _children[i]->nextCandidateDocument() );
  }
  return candidate;
}

void vank::infnet::PlusNode::annotate( vank::infnet::Annotator& annotator, lemur::api::DOCID_T documentID, vank::index::Extent &extent ) {
  std::vector<BeliefNode*>::iterator iter;
  annotator.add( this, documentID, extent );

  for( iter = _children.begin(); iter != _children.end(); iter++ ) {
    (*iter)->annotate( annotator, documentID, extent );
  }
}

double vank::infnet::PlusNode::maximumBackgroundScore() {
  std::vector<BeliefNode*>::iterator iter;
  double minimum = 0.0;

  for( iter = _children.begin(); iter != _children.end(); iter++ ) {
    minimum += (*iter)->maximumBackgroundScore();
  }

  return minimum;
}

double vank::infnet::PlusNode::maximumScore() {
  std::vector<BeliefNode*>::iterator iter;
  double maximum = 0.0;

  for( iter = _children.begin(); iter != _children.end(); iter++ ) {
    maximum += (*iter)->maximumScore();
  }

  return maximum;
}

vank::utility::greedy_vector<vank::api::ScoredExtentResult>& vank::infnet::PlusNode::score( lemur::api::DOCID_T documentID, vank::index::Extent &extent, int documentLength ) {
  std::vector<BeliefNode*>::iterator iter;
  double score = 0;
  bool scored = false;
  vank::utility::greedy_vector< vank::utility::greedy_vector<vank::api::ScoredExtentResult> > scores;
  for( iter = _children.begin(); iter != _children.end(); iter++ ) {
    const vank::utility::greedy_vector<vank::api::ScoredExtentResult>& childResults = (*iter)->score( documentID, extent, documentLength );
    scores.push_back(childResults);
  }
  int i = 0;
  for( iter = _children.begin(); iter != _children.end(); iter++ ) {
    double childScore = 0;
    vank::utility::greedy_vector<vank::api::ScoredExtentResult>& childResults = scores[i];
    for( size_t j=0; j<childResults.size(); j++ ) {
      scored = true;
      childScore += childResults[j].score;
    }
    i++;
    score += childScore;
  }

  _scores.clear();
  if (scored) {
    // dmf 12/03 if no child returned an extent, don't push one
    vank::api::ScoredExtentResult result(extent);
    result.score=score;
    result.document=documentID;
    _scores.push_back( result );
  }

  return _scores;
}

//
// hasMatch
//

bool vank::infnet::PlusNode::hasMatch( lemur::api::DOCID_T documentID ) {
  for( size_t i=0; i<_children.size(); i++ ) {
    if( _children[i]->hasMatch( documentID ) )
      return true;
  }
  return false;
}

//
// hasMatch
//

const vank::utility::greedy_vector<bool>& vank::infnet::PlusNode::hasMatch( lemur::api::DOCID_T documentID, const vank::utility::greedy_vector<vank::index::Extent>& extents ) {
  _matches.clear();
  _matches.resize( extents.size(), false );

  for( size_t i=0; i<_children.size(); i++ ) {
    const vank::utility::greedy_vector<bool>& kidMatches = _children[i]->hasMatch( documentID, extents );

    for( size_t j=0; j<kidMatches.size(); j++ ) {
      if( kidMatches[j] ) {
        _matches[j] = true;
      }
    }
  }

  return _matches;
}

//
// getName
//

const std::string& vank::infnet::PlusNode::getName() const {
  return _name;
}
