/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// PriorNode
//
// 6 July 2004 -- tds
//

#include "vank/PriorNode.hpp"
#include "vank/greedy_vector"
#include <math.h>
#include "vank/Annotator.hpp"
#include "vank/InferenceNetwork.hpp"

vank::infnet::PriorNode::PriorNode( const std::string& name,
                                     class InferenceNetwork& network,
                                     int listID ) :
  _name(name),
  _listID(listID),
  _network(network),
  _iterator(0)
{
}

vank::infnet::PriorNode::~PriorNode() {
}

lemur::api::DOCID_T vank::infnet::PriorNode::nextCandidateDocument() {
  return MAX_INT32;
}

bool vank::infnet::PriorNode::hasMatch( lemur::api::DOCID_T documentID ) {
  // priors don't match; they only boost or cut
  return false;
}

const vank::utility::greedy_vector<bool>& vank::infnet::PriorNode::hasMatch( lemur::api::DOCID_T documentID, const vank::utility::greedy_vector<vank::index::Extent>& extents ) {
  _matches.resize( extents.size(), false );
  return _matches;
}

const vank::utility::greedy_vector<vank::api::ScoredExtentResult>& vank::infnet::PriorNode::score( lemur::api::DOCID_T documentID, vank::index::Extent &extent, int documentLength ) {
  double score = -1e100;

  if( _iterator && !_iterator->finished() )
    score = _iterator->currentEntry()->score;

  _scores.clear();
  vank::api::ScoredExtentResult result(extent);
  result.score=score;
  result.document=documentID;
  _scores.push_back( result );
  return _scores;
}

void vank::infnet::PriorNode::annotate( class vank::infnet::Annotator& annotator, lemur::api::DOCID_T documentID, vank::index::Extent &extent ) {
  score( documentID, extent, extent.end );

  for( unsigned int i=0; i<_scores.size(); i++ ) {
    annotator.add( this, documentID, (vank::index::Extent &)_scores[i]);
  }
}

double vank::infnet::PriorNode::maximumScore() {
  return VANK_HUGE_SCORE;
}

double vank::infnet::PriorNode::maximumBackgroundScore() {
  return VANK_HUGE_SCORE;
}

const std::string& vank::infnet::PriorNode::getName() const {
  return _name;
}

void vank::infnet::PriorNode::indexChanged( vank::index::Index& index ) {
  _iterator = _network.getPriorIterator( _listID );
}
