/*==========================================================================
 * Copyright (c) 2004 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */


//
// QueryAnnotation
//
// 21 July 2004 -- tds
//

#include "vank/QueryAnnotation.hpp"
#include "vank/Walker.hpp"
#include <map>
#include <set>
#include <stack>
#include "vank/QuerySpec.hpp"
namespace vank
{
  namespace lang
  {
    class QueryNodeBuilder : public vank::lang::Walker {
    private:
      vank::api::QueryAnnotationNode* _root;
      std::map< vank::lang::Node*, vank::api::QueryAnnotationNode* > _nodes;
      std::stack< vank::api::QueryAnnotationNode* > _stack;

    public:
      QueryNodeBuilder() : _root(0) {}

      void defaultBefore( vank::lang::Node* n ) {
        std::map< vank::lang::Node*, vank::api::QueryAnnotationNode* >::iterator iter = _nodes.find(n);
        vank::api::QueryAnnotationNode* next = 0;

        if( iter != _nodes.end() ) {
          next = iter->second;
        } else {
          next = new vank::api::QueryAnnotationNode;
          next->name = n->nodeName();
          next->queryText = n->queryText();
          next->type = n->typeName();
        }

        if( _stack.size() ) {
          _stack.top()->children.push_back( next );
        }

        _stack.push( next );

        if( !_root ) {
          _root = next;
        }
      }

      void defaultAfter( vank::lang::Node* n ) {
        _stack.pop();
      }

      vank::api::QueryAnnotationNode* getRoot() {
        return _root;
      }
    };
  }
}


void delete_query_node( vank::api::QueryAnnotationNode* node, std::set<vank::api::QueryAnnotationNode*>& deleted ) {
  // query tree may be a dag, so we have to be careful
  deleted.insert(node);

  for( size_t i=0; i<node->children.size(); i++ ) {
    vank::api::QueryAnnotationNode* child = node->children[i];
    if( deleted.find(child) == deleted.end() )
      delete_query_node(node->children[i], deleted);
  }
}


vank::api::QueryAnnotation::QueryAnnotation() :
  _queryTree(0) {
}

vank::api::QueryAnnotation::~QueryAnnotation() {
  std::set<vank::api::QueryAnnotationNode*> deleted;
  delete_query_node(_queryTree, deleted);
  std::set<vank::api::QueryAnnotationNode*>::iterator item;
  for(item = deleted.begin(); item != deleted.end(); item++) {
    delete *item;
  }
}


vank::api::QueryAnnotation::QueryAnnotation( vank::lang::Node* queryRoot, vank::infnet::EvaluatorNode::MResults& annotations, std::vector<vank::api::ScoredExtentResult>& results ) {
  vank::lang::QueryNodeBuilder builder;
  queryRoot->walk(builder);
  _queryTree = builder.getRoot();
  _annotations = annotations;
  _results = results;
}

const vank::api::QueryAnnotationNode* vank::api::QueryAnnotation::getQueryTree() const {
  return _queryTree;
}

const std::vector<vank::api::ScoredExtentResult>& vank::api::QueryAnnotation::getResults() const {
  return _results;
}

const vank::infnet::EvaluatorNode::MResults& vank::api::QueryAnnotation::getAnnotations() const {
  return _annotations;
}
