/*==========================================================================
 * Copyright (c) 2005 University of Massachusetts.  All Rights Reserved.
 *
 * Use of the Lemur Toolkit for Language Modeling and Information Retrieval
 * is subject to the terms of the software license set forth in the LICENSE
 * file included with this software, and also available at
 * http://www.lemurproject.org/license.html
 *
 *==========================================================================
 */

//
// QueryParserFactory
//
// 30 Aug 2005 -- dmf
//

#include <sstream>

#include "vank/QueryParserFactory.hpp"
// additional parser/lexer types here
#include "vank/QueryLexer.hpp"
#include "vank/QueryParser.hpp"
#include "vank/NexiLexer.hpp"
#include "vank/NexiParser.hpp"

#include "lemur/Exception.hpp"

namespace vank
{
  namespace api
  {

    /* template wrapper class for antlr generated parser/lexer combination.
     */
    template<typename _lexType, typename _parseType>

    class _Wrapper : public QueryParserWrapper {
    public:
      _Wrapper(_lexType *lexer,
               _parseType *parser,
               std::istringstream* queryStream) :
        _lexer(lexer),_parser(parser), _queryStream(queryStream) {
      }

      virtual vank::lang::ScoredExtentNode *query() {
        return _parser->query();
      }

      virtual ~_Wrapper() {
        delete(_parser);
        delete(_lexer);
        delete(_queryStream);
      }
    private:
      _lexType *_lexer;
      _parseType *_parser;
      std::istringstream* _queryStream;

    };

    QueryParserWrapper *QueryParserFactory::get(const std::string &query,
                                                const std::string &parserType) {
      QueryParserWrapper *retval = 0;
      // need scope
      std::istringstream* queryStream = new std::istringstream(query);
      if (parserType == "vank") {
        vank::lang::QueryLexer *lexer = new vank::lang::QueryLexer ( *queryStream );
        vank::lang::QueryParser *parser = new vank::lang::QueryParser ( *lexer );
        // this step is required to initialize some internal
        // parser variables, since ANTLR grammars can't add things
        // to the constructor
        parser->init( lexer );
        lexer->init();
        QueryParserWrapper *retval = new _Wrapper<vank::lang::QueryLexer, vank::lang::QueryParser>(lexer, parser, queryStream);
        return retval;
      }
      else if (parserType == "nexi") {
        vank::lang::NexiLexer *lexer = new vank::lang::NexiLexer ( *queryStream );
        vank::lang::NexiParser *parser = new vank::lang::NexiParser ( *lexer );
        // this step is required to initialize some internal
        // parser variables, since ANTLR grammars can't add things
        // to the constructor
        parser->init( lexer );
        lexer->init();
        QueryParserWrapper *retval = new _Wrapper<vank::lang::NexiLexer, vank::lang::NexiParser>(lexer, parser, queryStream);
        return retval;
      }
      else {
        LEMUR_THROW(LEMUR_MISSING_PARAMETER_ERROR, "could not query parser for " + parserType);
      }
    }
  }
}
